import os
import sys
# stdin handle is -10
# stdout handle is -11
# stderr handle is -12

def getSize():
    if os.name == 'nt':
    	from ctypes import windll, create_string_buffer
        h = windll.kernel32.GetStdHandle(-12)
        csbi = create_string_buffer(22)
        res = windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)

        if res:
            import struct
            (bufx, bufy, curx, cury, wattr,
             left, top, right, bottom, maxx, maxy) = struct.unpack("hhhhHhhhhhh", csbi.raw)
            sizex = right - left + 1
            sizey = bottom - top + 1
        else:
            sizex, sizey = 80, 25 # can't determine actual size - return default values
        return sizex, sizey
        # print sizex, sizey
    elif os.name == 'posix':
        from fcntl import ioctl
        from termios import TIOCGWINSZ
        from array import array
    
        winsize = array("H", [0] * 4)
        try:
            ioctl(sys.stdout.fileno(), TIOCGWINSZ, winsize)
        except IOError:
            pass
        # return (winsize[1], winsize[0])[0]
        return (winsize[1], winsize[0])
    return 80

def getWidth():
    return getSize()[0]

def getHeight():
    return getSize()[1]

if __name__ == '__main__':
    print getSize()
    sizex = getWidth()
    print "_"*int(sizex)



#!/usr/bin/python2

import requests
import cfscrape
from bs4 import BeautifulSoup as bs
import argparse
import sys
import re
import clipboard
import platform
if platform.uname()[0] == 'Windows':
    import cmdw
    from idm import IDMan
import vping
import pywget
import os
pid = os.getpid()
import traceback
# import tracert
import termcolor
from make_colors import make_colors
import random
import time
import urlparse
from pydebugger.debug import debug
import configset
import threading
try:
    import __init__
    __version__ = __init__.__version__
    __test__ = __init__.__test__
    __build__ = __init__.__build__
    __platform__ = __init__.__platform__
    __email__ = __init__.__email__
except:
    pass
import cmdw
MAX = cmdw.getWidth()
MAX_TRY = 10
# EXIT=False
# NAME = ""
# url_result = ""
import traceback

ALL_SITES = ['drive.google', 'siotong', 'zonawibu', 'anonfile', 'isengsub', 'link.safelinkconverter', 'link.safelinkconverter.com']

class autogeneratelink(object):

    def __init__(self, link=None, proxy = None):
        super(autogeneratelink, self)
        self.link = link
        self.url = 'http://www.autogeneratelink.info/link.php'
        self.proxy = {}
        self.header = {}
        self.auto = False
        self.try_count = 0
        self.percent = 0
        self.thread = None
        self.MEMCACHED = False
        self.MEMCACHED_HOST = ''
        self.MEMCACHED_PORT = ''
        self.CONFIG = configset.configset()
        self.CONFIG.configname = os.path.join(os.path.dirname(__file__), 'aglink.ini')
        self.cf = cfscrape.create_scraper()
        if vping.vping(urlparse.urlparse(self.url).netloc.split('www.')[1]):
            pass
        else:
            print make_colors("No Internet Connection !!!", 'yellow', attrs= ['blink'])
            return
        self.choices = ['red', 'yellow', 'cyan', 'green', 'white', 'blue', 'magenta']
        self.set_memcached({'percent': 0, 'name': '', 'link': '',})
        if self.CONFIG.read_config('PROXY', 'server') and self.CONFIG.read_config('PROXY', 'port'):
            self.proxy.update(
                {
                    'http': 'http://%s:%s'%(self.CONFIG.read_config('PROXY', 'server'), self.CONFIG.read_config('PROXY', 'port')), 
                    'https': 'https://%s:%s'%(self.CONFIG.read_config('PROXY', 'server'), self.CONFIG.read_config('PROXY', 'port'))
                })

    def test_proxy(self):
        if self.proxy:
            try:
                a = requests.request('GET', self.url, timeout=5, proxies=self.proxy)
                print "STATUS:", a.status_code
            except:
                if os.getenv('DEBUG'):
                    traceback.format_exc()
                check_proxy = urlparse.urlparse(self.proxy.get('http')).netloc
                print make_colors('Checking proxy for: ', 'lightyellow') + make_colors("%s"%(check_proxy), 'black', 'lightcyan') + " " + make_colors("ERROR", 'lightwhite', 'lightred')

    def set_memcached(self, dict_data, host = '127.0.0.1', port = '11211', debug = 0):
        if self.MEMCACHED_HOST:
            host = self.MEMCACHED_HOST
        if self.MEMCACHED_PORT:
            port = self.MEMCACHED_PORT
        try:
            import memcache
            mc = memcache.Client([host + ":" + port], debug = debug)
            for i in dict_data:
                mc.set(i, dict_data.get(i))
        except ImportError:
            print "memcache not installed !"
            return False

    def get_memcached(self, key, host = '127.0.0.1', port = '11211', debug = 0):
        values = []
        if self.MEMCACHED_HOST:
            host = self.MEMCACHED_HOST
        if self.MEMCACHED_PORT:
            port = self.MEMCACHED_PORT
        try:
            import memcache
            mc = memcache.Client([host + ":" + port], debug = debug)
            if isinstance(key, list) or isinstance(key, tuple):
                for i in key:
                    value = mc.get(i)
                    values.append(value)
            else:
                values = mc.get(key)
            return values

        except ImportError:
            print "memcache not installed !"
            return False

    def percent_memcached(self, percent, host = '127.0.0.1', port = '11211', debug = 0):
        if self.MEMCACHED_HOST:
            host = self.MEMCACHED_HOST
        if self.MEMCACHED_PORT:
            port = self.MEMCACHED_PORT
        try:
            import memcache
            mc = memcache.Client([host + ":" + port], debug = debug)
            mc.set('agl_percent', percent)
        except ImportError:
            print "memcache not installed !"
            return False
    def get_percent_memcached(self, host = '127.0.0.1', port = '11211', debug = 0):
        if self.MEMCACHED_HOST:
            host = self.MEMCACHED_HOST
        if self.MEMCACHED_PORT:
            port = self.MEMCACHED_PORT
        try:
            import memcache
            mc = memcache.Client([host + ":" + port], debug = debug)
            percent = mc.get('agl_percent')
            return percent
        except ImportError:
            print "memcache not installed !"
            return False

    def percent_progress(self, percent, memcached = False):
        #print "memcache =", memcached
        #debug(memcached = memcached)
        if not self.percent >= 100:
            self.percent = percent
        if memcached:
            if self.MEMCACHED:
                self.percent_memcached(percent)
        else:
            if self.MEMCACHED:
                self.percent_memcached(percent)
        #print "percent =", percent
        return self.percent

    def getVersion(self, ):
        try:
            return __version__ + "." + __test__
        except:
            pass

    def cek_error(self, c, respon=None):
        try:
            if c == 'Never Give Up ! Generate again and try it up to 10x Generate Link !':
                # print termcolor.colored('Never Give Up ! Generate again and try it up to 10x Generate Link !', 'red', 'on_yellow', attrs= ['bold', 'blink'])
                print make_colors('Never Give Up ! Generate again and try it up to 10x Generate Link !', 'red', 'on_yellow', attrs= ['bold', 'blink'])
                return True, 'Never Give Up ! Generate again and try it up to 10x Generate Link !'
            elif c == 'Link Dead! or Host is temporarily down! Generate again after 5 minutes!':
                # print termcolor.colored('Link Dead! or Host is temporarily down! Generate again after 5 minutes!', 'white', 'on_red', attrs= ['bold', 'blink'])
                print make_colors('Link Dead! or Host is temporarily down! Generate again after 5 minutes!', 'white', 'on_red', attrs= ['bold', 'blink'])
                return True, 'Link Dead! or Host is temporarily down! Generate again after 5 minutes!'
            elif c == 'Link Dead!':
                # print termcolor.colored('Link Dead! or Host is temporarily down! Generate again after 5 minutes!', 'white', 'on_red', attrs= ['bold', 'blink'])
                print make_colors('Link Dead! or Host is temporarily down! Generate again after 5 minutes!', 'white', 'on_red', attrs= ['bold', 'blink'])
                return True, 'Link Dead! or Host is temporarily down! Generate again after 5 minutes!'
            elif c == 'Generate Failed!':
                if respon:
                    if len(respon) == 2:
                        # print termcolor.colored('Generate Failed!', 'white', 'on_red', attrs= ['bold', 'blink']) + termcolor.colored(' ', 'white') + termcolor.colored('~', 'yellow') + termcolor.colored(' ', 'white') + termcolor.colored(respon[0], 'yellow') + termcolor.colored(' ', 'white') + termcolor.colored('~', 'yellow')  + termcolor.colored(' ', 'white') + termcolor.colored(respon[1], 'cyan')
                        print make_colors('Generate Failed!', 'white', 'on_red', attrs= ['bold', 'blink']) + make_colors(' ', 'white') + make_colors('~', 'yellow') + make_colors(' ', 'white') + make_colors(respon[0], 'yellow') + make_colors(' ', 'white') + make_colors('~', 'yellow')  + make_colors(' ', 'white') + make_colors(respon[1], 'cyan')
                    elif len(respon) == 1 :
                        # print termcolor.colored('Generate Failed!', 'white', 'on_red', attrs= ['bold', 'blink']) + termcolor.colored(' ', 'white') + termcolor.colored('~', 'yellow') + termcolor.colored(' ', 'white') + termcolor.colored(respon[0], 'yellow')
                        print make_colors('Generate Failed!', 'white', 'on_red', attrs= ['bold', 'blink']) + make_colors(' ', 'white') + make_colors('~', 'yellow') + make_colors(' ', 'white') + make_colors(respon[0], 'yellow')
                return True, 'Generate Failed!'
            else:
                return False, ''
        except:
            if debug.DEBUG:
                traceback.format_exc()
            return False, ''
        return False, ''

    def get_req(self, link, headers = {}, proxy = {}):
        '''
            set corrent generate url
            parameter:
                link: (str) url
                proxy: (instance) ~ return of setProxy()
            return: (str) ~ requests.content
        '''
        debug()
        if not proxy:
            proxy = self.proxy
        debug(proxy=proxy)
        # print "proxy 0       =", proxy
        # print "type(proxy 0) =", type(proxy)
        if isinstance(proxy, dict):
            proxy_list = []
            for i in proxy:
                # print "i =",i
                # print "proxy.get(i) =", proxy.get(i)
                proxy_list.append(proxy.get(i))
            proxy = proxy_list
        
        # print "proxy 1 =", proxy
        # global EXIT
        name = ""
        url_result = ""
        nx = 1
        while 1:
            try:
                proxy_1 = {}
                status_code = requests.request('GET', self.url, timeout=5, proxies=proxy_1).status_code
                # if nx >= 1:
                #     print "\n"
                break
            except:
                time.sleep(1)
                sys.stdout.write(".")
                nx += 1
                proxy_1 = self.setProxy(proxy)
        debug(status_code=status_code)
        if status_code  == 200:
            proxy = {}
        else:
            proxy = self.setProxy(proxy)
        debug(print_function_parameters= True)
        params = {
            'link': link,
            'token': 'agl',
        }

        debug(params = params)

        r = None
        c_connect = 0
        c = None
        error = ''
        while 1:
            try:
                r = requests.get(self.url, params= params, proxies=proxy)
                debug(r_url = r.url)
                # print "\n"
                break
            except:
                if c_connect == 0:
                    print "connecting ."
                    c_connect = 1
                sys.stdout.write("+")
                if debug.DEBUG:
                    traceback.format_exc()
        if r:
            debug(r_text=r.text)
            b = bs(r.text, 'lxml')
            debug(b = b)
        else:
            print make_colors("FATAL CONNECTION !", 'white', 'red', ['blink', 'bold'])
            sys.exit(0)

        try:
            respon = re.split(" ", b.text)
            for i in respon:
                if str(i).strip() == '':
                    respon.remove(i)
            if respon[1] == '//':
                name = respon[0]
            debug(respon=respon)
            debug(respon_0=respon[0])
            if len(respon) > 1:
                debug(respon_1=respon[1])
                if len(respon) > 2:
                    debug(respon_2=respon[2])
            respon1 = re.findall(r'(https?://[^\s]+)', respon[2])
            if len(respon1) > 1:
                respon1 = respon1[0]
            if respon1:
                url_result = respon1
            debug(respon1=respon1)
            respon2 = re.split(" ", respon[0])
            debug(respon2=respon2)
            if respon1 in respon2:
                respon2.remove(respon1)
            respon2 = " ".join(respon2)
            debug(respon2=respon2)
            respon_send = [respon2, respon1]
            # respon_send = [respon[0], respon1]

            c =  b.find('b').text
            debug(c = c)
            error = self.cek_error(c, respon=respon_send)
            debug(error = error)
            # if error[0]:
            #     EXIT = True
            #     debug(EXIT=EXIT)
        except:
            if os.getenv('DEBUG_EXTRA'):
                traceback.format_exc()
            else:
                traceback.format_exc(print_msg=False)        
        debug(r=r)
        debug(error=error)
        # debug(EXIT=EXIT)
        if r.status_code == 200:
            # if EXIT:
            #     return False, False, error
            if error and len(error) > 1:
                debug(error=error)
                debug(error_0=error[0])
                if error[0] == True:
                    debug("error_0_False")
                    return False, False, error
            
            debug(url_result=url_result)
            if not url_result:
                try:
                    c = b.find('a', target = re.compile('blank')).get('href')
                    debug(c = c)
                except:
                    c = None
                    traceback.format_exc(print_msg = False)
            else:
                c = url_result
            debug(c=c)
            if not name:
                try:
                    name = b.find('b').text.split('//')[0].strip()
                    self.percent_progress(50, self.MEMCACHED)
                except:
                    if os.getenv('DEBUG_EXTRA'):
                        traceback.format_exc()
                    name = None
            else:
                if error and len(error) > 1:
                    if error[0]:
                        name = None
            debug(name = name)
            self.percent_progress(70, self.MEMCACHED)
            if not name:
                return c, None, error
            else:
                return c, name.encode('utf-8'), error
        else:
            return None, None, error

    def support(self, proxy = None):
        if not proxy:
            proxy = self.proxy
        g = requests.get(self.url, proxies = proxy)
        s = bs(g.text, 'lxml')
        b = s.find('textarea', {'class': 'form-control'})
        return b.get('placeholder')

    def download(self, url, path = ".", output = None, referrer = None, postdata = None, cookies = None, username = None, password = None, confirm = False, wget = False):
        path = os.path.abspath(path)
        if os.path.isfile(path):
            path = os.path.dirname(path)
            output = os.path.basename(path)
        if 'win' in sys.platform:
            try:
                import idm
                IDM = idm.IDMan()
                IDM.download(url, path, output)
            except:
                traceback.format_exc()
                filename = pywget.download(url, path)
                return filename
        elif wget:
            filename = pywget.download(url, path)
            return filename            
        else:
            filename = pywget.download(url, path)
            return filename

    def generate(self, link, clip=None, quality=None, verbosity=None, support= False, direct_download=None, download_path=".", pcloud = False, pcloud_username = None, pcloud_password = None, pcloud_folderid = '0', pcloud_rename = None, pcloud_foldername = None, proxy = None, fast = False, bypass_regenerate = False, cliped = False, name = None, wget = False, show_debug = False, no_return=False, use_proxy=False, auto=False):
        global ALL_SITES
        if show_debug:
            os.environ.update({'DEBUG':'1'})
        if not os.path.isdir(download_path):
            os.makedirs(download_path)
        if show_debug:
            debug.DEBUG = 1
        if pcloud_rename and not name:
            name = pcloud_rename
        if name and not pcloud_rename:
            pcloud_rename = name
        debug(link0 = link)
        if cliped:
            link = clipboard.paste()
        debug(link1 = link)
        choices = ['red', 'yellow', 'cyan', 'green', 'white', 'blue', 'magenta']
        if not proxy:
            proxy = self.proxy
        if support:
            print "\n"
            print self.support()
            print "\n"

        if link == None:
            if self.link == None:
                return False, None
            else:
                pass

        if 'youtu' in link:
            self.youtube(link, direct_download, download_path, True, pcloud, pcloud_username, pcloud_password, pcloud_folderid, pcloud_rename, pcloud_foldername)
        else:
            debug(link2 = link)
            if self.get_netloc(link) == 'siotong' or self.get_netloc(link) == 'coeg' or self.get_netloc(link) == 'telondasmu' or self.get_netloc(link) == 'siherp' or self.get_netloc(link) == 'greget' or self.get_netloc(link) == 'tetew' or self.get_netloc(link) == 'anjay':
                link = self.siotong(link)
                debug(link = link)
            elif self.get_netloc(link) == 'zonawibu':
                link = self.zonawibu(link)
            elif self.get_netloc(link) == 'anonfile':
                link = self.anonfile(link)
            elif self.get_netloc(link) == 'isengsub':
                link = self.isengsub(link)
            elif self.get_netloc(link) == 'mediafire':
                link = self.mediafire(link)
            elif urlparse.urlparse(link).netloc == 'link.safelinkconverter.com':
                link = self.safelinkconverter(link)
            elif urlparse.urlparse(link).netloc == 'ourabeauty.info':
                link = self.ourabeauty(link)
            else:
                self.percent_progress(30, self.MEMCACHED)
            if self.get_netloc(link) in ALL_SITES:
                print "Re:Generate [ALL_SITES] ..."
                return self.generate(link, clip, quality, verbosity, support, direct_download, download_path, pcloud, pcloud_username, pcloud_password, pcloud_folderid, pcloud_rename, pcloud_foldername, proxy, fast, bypass_regenerate, cliped, name, wget, show_debug, no_return)
            if not self.get_netloc(link) == 'anonfile' and not self.get_netloc(link) == 'isengsub' and not 'mediafire' in link:
                a, out_name, error = self.get_req(link)
            else:
                if clip:
                    clipboard.copy(str(link))
                if direct_download and not pcloud:
                    try:
                        self.percent_progress(99, self.MEMCACHED)
                        idm = IDMan()
                        idm.download(link, download_path)
                        self.percent_progress(100, self.MEMCACHED)
                    except:
                        self.percent_progress(99, self.MEMCACHED)
                        pywget.download(link, out= download_path)
                        self.percent_progress(100, self.MEMCACHED)   
                elif direct_download and pcloud:
                    if os.path.isfile(download_path):
                        download_path = os.path.dirname(download_path)
                        name = os.path.basename(download_path)                    
                    else:
                        name = None
                    print make_colors('Upload to PCloud and download it...', 'white', 'magenta')
                    self.pcloud(link, pcloud_username, pcloud_password, name, pcloud_folderid, pcloud_foldername, direct_download, download_path, use_proxy)
                elif pcloud and not direct_download:
                    print make_colors('Upload to PCloud and download it...', 'white', 'magenta')
                    self.pcloud(link, pcloud_username, pcloud_password, name, pcloud_folderid, pcloud_foldername, direct_download, download_path, use_proxy)
                print make_colors('RESULT GENERATED:', 'lightwhite', 'lightmagenta') + " " + make_colors(link, 'lightgreen')
                return link
            debug(a = a)
            debug(out_name = out_name)
            if out_name and not isinstance(out_name, bool):
                if out_name.strip() == "[Attention] File too big! when allowed only 1.1 Gb on Mega.co.nz ! (Because our server disconnect after 1,1Gb)":
                        error = True, "[Attention] File too big! when allowed only 1.1 Gb on Mega.co.nz ! (Because our server disconnect after 1,1Gb)"
            debug(error=error)
            # print "error =", error
            if isinstance(error, tuple) or isinstance(error, list):
                if error[0]:
                    debug('ERROR 001')
                    if auto:
                        return self.generate(link, clip, quality, verbosity, support, direct_download, download_path, pcloud, pcloud_username, pcloud_password, pcloud_folderid, pcloud_rename, pcloud_foldername, proxy, fast, bypass_regenerate, cliped, name, wget, show_debug, no_return, use_proxy, auto)
                    elif self.auto:
                        return self.generate(link, clip, quality, verbosity, support, direct_download, download_path, pcloud, pcloud_username, pcloud_password, pcloud_folderid, pcloud_rename, pcloud_foldername, proxy, fast, bypass_regenerate, cliped, name, wget, show_debug, no_return, use_proxy, self.auto)
                    q_err = raw_input(make_colors("Re:Generate [y/n/a]: "))
                    if q_err.lower() == 'y':
                        return self.generate(link, clip, quality, verbosity, support, direct_download, download_path, pcloud, pcloud_username, pcloud_password, pcloud_folderid, pcloud_rename, pcloud_foldername, proxy, fast, bypass_regenerate, cliped, name, wget, show_debug, no_return, use_proxy, auto)
                    elif q_err.lower() == 'a':
                        return self.generate(link, clip, quality, verbosity, support, direct_download, download_path, pcloud, pcloud_username, pcloud_password, pcloud_folderid, pcloud_rename, pcloud_foldername, proxy, fast, bypass_regenerate, cliped, name, wget, show_debug, no_return, use_proxy, True)   
                    else:
                        return error[1]
            
            if isinstance(a, list):
                debug(a=a)
                if not len(a[0]) > 5:
                    print make_colors(name, 'white', 'red', ['blink'])
                    qr = raw_input(make_colors('Re-Generate again', 'white', 'blue') + " " + make_colors('[Y/N]', 'white', 'red') + ': ')
                    if str(qr).lower() == 'n':
                        self.percent_progress(100, self.MEMCACHED)
                        sys.exit(0)
                    elif str(qr).lower() == 'y':
                        self.percent_progress(0, self.MEMCACHED)
                        return self.generate(link, clip, quality, verbosity, support, direct_download, download_path, pcloud, pcloud_username, pcloud_password, pcloud_folderid, pcloud_rename, pcloud_foldername, proxy, fast)
                    else:
                        self.percent_progress(100, self.MEMCACHED)
                        sys.exit('SAVE EXIT')

            if out_name == 'Never Give Up ! Generate again and try it up to 10x Generate Link !' or a == 'Never Give Up ! Generate again and try it up to 10x Generate Link !' or error == 'Never Give Up ! Generate again and try it up to 10x Generate Link !':
                if bypass_regenerate:
                    return a, out_name
                else:
                    qr = raw_input(make_colors('Re-Generate again', 'white', 'blue') + " " + make_colors('[Y/N]', 'white', 'red') + ': ')
                    if str(qr).lower() == 'n':
                        self.percent_progress(100, self.MEMCACHED)
                        sys.exit(0)
                    elif str(qr).lower() == 'y':
                        self.percent_progress(0, self.MEMCACHED)
                        return self.generate(link, clip, quality, verbosity, support, direct_download, download_path, pcloud, pcloud_username, pcloud_password, pcloud_folderid, pcloud_rename, pcloud_foldername, proxy, fast)
                    else:
                        self.percent_progress(100, self.MEMCACHED)
                        sys.exit('SAVE EXIT')
            # print "a =", a
            if a:
                if isinstance(a, list):
                    a = a[0]
                debug(a=a)
                filename = None
                disposition = None
                try:
                    disposition = requests.request('GET', a, stream=True, proxies=proxy, timeout=10).headers.get('Content-Disposition')
                except:
                    pass
                if disposition:                    
                    filename = disposition.split('attachment; filename=')
                    debug(filename=filename, debug=True)
                    if len(filename) > 1:
                        filename = filename[1]
                    filename = str(filename).replace('AutoGenerateLink_%5', '')
                    filename = str(filename).replace('AutoGenerateLink_', '')
                    filename = str(filename).replace('www.gigapurbalingga.net%5d', '')
                    filename = str(filename).replace('www.gigapurbalingga.net', '')
                    filename = str(filename).replace('-SAMEHADAKU.TV', '')
                    filename = str(filename).replace('-samehadaku.tv', '')
                    filename = str(filename).replace('b_', '')
                    debug(filenamex=filename)
                if filename:
                    name = filename
                    out_name = filename
                self.percent_progress(90, self.MEMCACHED)
                debug(name=name)
                if not name:
                    if out_name:
                        name = out_name
                    else:
                        name = ''
                name = os.path.basename(name)
                # if not os.path.isdir(download_path):
                #     download_path, name = os.path.split(download_path)
                #     out_name = name
                # print termcolor.colored('GENERATED         : ', 'white', 'on_red') + termcolor.colored(a, 'white', 'on_red')
                print make_colors('GENERATED         : ', 'white', 'on_red') + make_colors(a, 'white', 'on_red')
                # print termcolor.colored('NAME              : ', 'white', 'on_green') + termcolor.colored(out_name, 'white', 'on_blue')
                print make_colors('NAME              : ', 'white', 'on_green') + make_colors(out_name, 'white', 'on_blue')
                # print termcolor.colored('DOWNLOAD NAME     : ', 'white', 'on_cyan') + termcolor.colored(name, 'white', 'on_blue')
                print make_colors('DOWNLOAD NAME     : ', 'white', 'on_cyan') + make_colors(name, 'white', 'on_blue')
                if out_name == 'Generate Failed!':
                    self.percent_progress(100, self.MEMCACHED)
                    sys.exit('FAILED!')
                # if not name:
                #     name = out_name
                # if name:
                #     name = os.path.basename(name)
                if pcloud and not direct_download:
                    if os.path.isfile(download_path):
                        download_path = os.path.dirname(download_path)
                        name = os.path.basename(download_path)
                    print make_colors('Upload to PCloud ...', 'white', 'magenta')
                    if show_debug:
                        try:
                            os.environ.pop('DEBUG')
                        except:
                            pass
                    self.pcloud(a, pcloud_username, pcloud_password, name, pcloud_folderid, pcloud_foldername, False, download_path, use_proxy)
                if pcloud and direct_download:
                    if os.path.isfile(download_path):
                        download_path = os.path.dirname(download_path)
                        name = os.path.basename(download_path)                    
                    print make_colors('Upload to PCloud and download it...', 'white', 'magenta')
                    if show_debug:
                        try:
                            os.environ.pop('DEBUG')
                        except:
                            pass
                    self.pcloud(a, pcloud_username, pcloud_password, name, pcloud_folderid, pcloud_foldername, direct_download, download_path, use_proxy)
                if clip:
                    self.percent_progress(92, self.MEMCACHED)
                    if name:
                        clipboard.copy(str(name))
                    if a:
                        clipboard.copy(str(a))
                    self.percent_progress(100, self.MEMCACHED)
                if direct_download and not pcloud:
                    self.percent_progress(95, self.MEMCACHED)
                    if os.path.isfile(download_path):
                        download_path = os.path.dirname(download_path)
                        name = os.path.basename(download_path)
                    dict_data = {
                        'name': name,
                        'link': a,
                    }
                    self.set_memcached(dict_data)                    
                    print make_colors('Download it...', 'white', 'blue')
                    #filename = pywget.download(a.get('href'), download_path)
                    #if 'youtu' in link:
                        #name = self.download(str(youtube_list.get(int(q))), download_path, wget = wget)
                    try:
                        self.percent_progress(99, self.MEMCACHED)
                        idm = IDMan()
                        idm.download(a, download_path, name)
                        self.percent_progress(100, self.MEMCACHED)
                    except:
                        self.percent_progress(99, self.MEMCACHED)
                        pywget.download(a, out= os.path.join(download_path, name))
                        self.percent_progress(100, self.MEMCACHED)
                    return a, name
                self.percent_progress(100, self.MEMCACHED)
                dict_data = {
                    'name': name,
                    'link': a,
                }
                self.set_memcached(dict_data)
                if show_debug:
                    try:
                        os.environ.pop('DEBUG')
                    except:
                        pass
                if not no_return:
                    return a, name

    def get_netloc(self, url):
        if "www." in url:
            debug(urlparse0 = urlparse.urlparse(url))
            netloc = urlparse.urlparse(url).netloc.split(".", 2)[1]
            debug(netloc0 = netloc)
            return netloc
        else:
            debug(urlparse1 = urlparse.urlparse(url))
            netloc = urlparse.urlparse(url).netloc.split(".", 1)[0]
            debug(netloc1 = netloc)
            return netloc

    def youtube(self, url, direct_download = False, download_path = ".", interactive = True, pcloud = False, pcloud_username = None, pcloud_password = None, pcloud_folderid = '0', pcloud_rename = None, pcloud_foldername = None):
        youtube_list = {}
        self.header.update({
            'Accept' : '*/*', 
            'Accept-Encoding' : 'gzip, deflate',
            'Accept-Language' : 'en-US,en;q=0.5',
            'Connection' : 'keep-alive',
            'Host' : 'www.autogeneratelink.us',
            'Referer' : 'http://www.autogeneratelink.us/', 
            'User-Agent' : 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:52.9) Gecko/20100101 Goanna/3.4 Firefox/52.9 PaleMoon/27.6.2',
            'X-Requested-With' : 'XMLHttpRequest'
        })
        headers = self.header
        debug(headers = headers)
        all_video = {}
        all_audio = {}
        n = 1
        m = 1        
        if 'youtu' in url:
            details = {}
            #a0 = self.get_req(url, headers)
            #http://www.autogeneratelink.us/link.php?link=https%3A%2F%2Fwww.youtube.com%2Fwatch%3Fv%3DUhqwlP_WgF4&token=agl
            params = {      
                'link': url,
                'token': 'agl',
            }
            debug(self_url=self.url)
            #URL = self.url + '/link.php?'
            # debug(URL=URL)
            #debug(headers = headers)
            a0 = requests.get(self.url, params = params, headers = headers, proxies=self.proxy)
            a0 = requests.get(self.url, params = params, proxies=self.proxy)
            debug(a0_content=a0.content)
            debug(FULL_URL = a0.url)
            a1 = bs(a0.content, 'lxml')
            debug(a1 = a1)
            #a1 = a11.find("li", {'class': 'list-group-item',})
            #debug(a1 = a1)
            #name = a1.find_next("b")
            try:
                name = a1.find("b").text
            except:
                print make_colors('SERVER ERROR !', 'white', 'lightred')
                if os.getenv('debug'):
                    import traceback
                    traceback.format_exc()
                sys.exit(0)
            debug(name = name)
            #c0 = a1.find_next('span', {'class': 'list-group-item',}).find_next('table')
            c0 = a1.find('table')
            debug(c0 = c0)
            if not c0:
                print (make_colors("INVALID URL !", 'lightwhite', 'lightred', attrs=['blink']))
                sys.exit(1)
            c1 = c0.find_all_next('tr')
            debug(c1 = c1)
            for i in c1:
                c2 = i.find_all_next('b')
                debug(c2 = c2)
                details.update({c2[0].text: c2[1].text,})
                debug(details = details)
            debug(details = details)

            d0 = a1.find_all('a', {'class': 'list-group-item',})
            debug(d0 = d0)
            n = 1
            for i in d0:
                i_quality = []
                a_quality = re.split("Video only|Video| only|Audio only|Audio|:| \| |\(|\)", str(i.text).strip())
                debug(a_quality = a_quality)
                for x in a_quality:
                    if not str(x).strip() == '':
                        i_quality.append(str(x).strip())
                debug(i_quality = i_quality)
                video = False
                audio = False
                video_quality = ''
                audio_quality = ''
                size = ''
                if len(i_quality) == 3:
                    video = True
                    audio = True
                    debug(video = video, audio = audio)
                    video_quality = i_quality[0]
                    audio_quality = i_quality[1]
                    size = i_quality[2]
                    debug(video_quality)
                    debug(audio_quality)
                    debug(size = size)
                else:
                    if 'Video only' in i.text:
                        video = True
                        audio = False
                        debug(video = video, audio = audio)
                        video_quality = i_quality[0]
                        size = i_quality[1]
                        debug(video_quality)
                        debug(size = size)                    
                    elif 'Audio only' in i.text:
                        video = False
                        audio = True
                        debug(video = video, audio = audio)
                        audio_quality = i_quality[0]
                        size = i_quality[1]
                        debug(audio_quality)
                        debug(size = size)

                youtube_list.update(
                    {
                        n: {
                            'title': i.text.strip(),
                            'url': i.get('href'),
                            'video': video,
                            'audio': audio,
                            'video_quality': video_quality,
                            'audio_quality': audio_quality,
                            'size': size,
                            },
                    }
                )
                n += 1
            debug(youtube_list = youtube_list)
            for i in youtube_list:
                if not youtube_list.get(i).get('video'):
                    i_video = ''
                if not youtube_list.get(i).get('audio'):
                    i_audio = ''
                print str(i) + ". " + youtube_list.get(i).get('title') + " (" + youtube_list.get(i).get('video_quality') + "~" + youtube_list.get(i).get('audio_quality') + ")"

            if not interactive:
                return youtube_list
            q = raw_input("Select Number: ")
            #if not quality:
                #q = raw_input("Select Number: ")
            #else:
                #q = quality
                #if len(youtube_list) < int(q):
                    #q = raw_input("Select Number: ")
                #else:
                    #q_result = {'audio': [], 'video': [],}
                    #for i in youtube_list:
                        #if str(q) in i.get('video_quality'):
                            #q_result.get('audio').append()
            if q == '' or q == ' ' or q == None:
                sys.stdout.write("  You Not select any number !")
                sys.exit(0)
            debug(URL_SELECTED = str(youtube_list.get(int(q)).get('url')))
            try:
                if isinstance(int(q), int) and int(q) > 0:
                    try:
                        clipboard.copy(str(youtube_list.get(int(q)).get('url')))
                    except:
                        traceback.format_exc()
                    if direct_download:
                        #filename = pywget.download(
                            #str(youtube_list.get(int(q))), download_path)
                        filename = self.download(str(youtube_list.get(int(q)).get('url')), download_path)
                        return str(youtube_list.get(int(q))), filename
                    if pcloud:
                        self.pcloud(str(youtube_list.get(int(q)).get('url')), pcloud_username, pcloud_password, None, pcloud_folderid, pcloud_foldername)                    
                    return str(youtube_list.get(int(q)).get('url')), None
            except:
                import traceback
                print "ERROR:"
                print traceback.format_exc()
        return youtube_list
        #sys.exit('still development ............... :)')
        # elif "play.google.com" in url:
        #     a = soup.find('a')
        #     return a.get('href'), None

    def safelinkconverter(self, url):
        a = self.cf.get(url)
        b = bs(a.content, 'lxml')
        URL1 = b.find('div', {'id':'m'}).find('a').get('href')
        debug(URL1=URL1)
        a1 = self.cf.get(URL1)
        b1 = bs(a1.content, 'lxml')
        URL2 = b1.find('div', {'class':"redirect_url"}).find_next('div').get('onclick')
        debug(URL2=URL2)
        URL = re.findall('https.*?\.html',URL2)
        debug(URL=URL)
        return URL[0]

    def anonfile(self, url):
        a = self.cf.get(url)
        b = bs(a.content, 'lxml')
        div_download_wrapper = b.find('div', id='download-wrapper')
        if div_download_wrapper:
            download_url = div_download_wrapper.find('a', id='download-url').get('href')
            debug(download_url=download_url)
        return download_url

    def ourabeauty(self, url):
        # http://ourabeauty.info/?id=R3R3ejRJOWNoQUQ4M2VkVUhRb2hodWNTcVJyQzMwUCt6cDI0V2FDZnhNWmlvNm56c1lzb016Tlh1UFBiemU3Vg==
        a = self.cf.get(url)
        b = bs(a.content, 'lxml')
        form = b.find('div', {'class':'humancheck'}).find('form')
        get_id = form.find('input').get('value')
        action = form.get('action')
        debug(get_id=get_id)
        a1 = self.cf.post(action, data={'get':get_id})
        b1 = bs(a1.content, 'lxml')
        c1 = b1.find('head').find_all('script', {'type':'text/javascript'})[-1]
        debug(c1=c1)
        # print "str(c1) =", str(c1)
        c2 = re.findall('changeLink.*?;', str(c1))
        debug(c2=c2)
        if c2:
            c3 = re.split("//|'|;", c2[0])[-3]
            debug(c3=c3)
            print make_colors("LINK", 'lw', 'lm') + " : " + make_colors('https://' + c3, 'lw', 'lb')
            return 'https://' + c3
        return False
        
    
    def isengsub(self, url):
        a = self.cf.get(url)
        b = bs(a.content, 'lxml')
        div_download_wrapper = b.find('div', {'class':'pusat'}).find('div', {'align':'center'})
        if div_download_wrapper:
            download_url = div_download_wrapper.find('a').get('href')
            debug(download_url=download_url)
        return download_url

    def mediafire(self, url):
        a = self.cf.get(url)
        b = bs(a.content, 'lxml')
        div_download_link = b.find('div', id='download_link')
        debug(div_download_link=div_download_link)
        if div_download_link:
            download_url = div_download_link.find('a', {'class':'input'}).get('href')
            debug(download_url=download_url)
            clipboard.copy(str(download_url))
        return download_url

    def youtube1(self, url):
        URL = 'http://api.w3hills.com/youtube/get_video_info'
        params = {
            'video_id': '',
            'token': '',
        }

    def siotong(self, url, verbose = False):
        '''
            generate url containt words "siotong" or "coeg"
            parameter:
                url = (str) ~ first url given
            return:
                str
                format: url
        '''
        while 1:
            try:                
                req = requests.get(url)
                break
            except:
                pass
        if verbose:
            os.environ.update({'DEBUG': "1"})
        if self.get_netloc(req.url) == 'siotong' or self.get_netloc(req.url) == 'coeg' or self.get_netloc(req.url) == 'telondasmu' or self.get_netloc(req.url) == 'siherp' or self.get_netloc(req.url) == 'greget' or self.get_netloc(req.url) == 'tetew' or self.get_netloc(req.url) == 'anjay':
            a = bs(req.content, 'lxml')

            b0 = a.find('div', {'class': 'download-link',})
            if b0:
                b = b0.find('a').get('href')  #get siotong
                debug(b_1 = b)
            else:
                b0 = a.find('div', {'class': "col-sm-12",})
                if b0:
                    b = b0.find_next('p', {'style': 'text-align:center;',}).find('a').get('href')
                    debug(b_2 = b)

            debug(b = b)
            if self.get_netloc(b) == 'siotong' or self.get_netloc(b) == 'coeg' or self.get_netloc(b) == 'telondasmu' or self.get_netloc(b) == 'siherp' or self.get_netloc(b) == 'greget' or self.get_netloc(req.url) == 'tetew' or self.get_netloc(req.url) == 'anjay':
                self.percent_progress(10, self.MEMCACHED)
                if verbose or str(os.getenv('DEBUG')) == '1':
                    print make_colors('re-generate siotong: ', 'white', 'red', ['blink']) + make_colors(str(b), 'blue', 'yellow') + " ..."
                return self.siotong(b)
        else:
            debug(self_get_netloc_req_url = self.get_netloc(req.url))
            debug(return_url = req.url)
            self.percent_progress(50, self.MEMCACHED)
            return req.url

    def zonawibu(self, url, verbose = False):
        '''
            generate url containt words "zonawibu"
            parameter:
                url = (str) ~ first url given
            return:
                str
                format: url
        '''
        req = requests.get(url)
        if verbose:
            os.environ.update({'DEBUG': "1"})
        if self.get_netloc(req.url) == 'zonawibu':
            a = bs(req.content, 'lxml')
            b = a.find('div', {'class': 'notify',}).find('a').get('href')  #get siotong
            debug(b = b)
            if self.get_netloc(b) == 'zonawibu':
                if verbose or str(os.getenv('DEBUG')) == '1':
                    print make_colors('re-generate siotong: ', 'white', 'red', ['blink']) + make_colors(str(b), 'blue', 'yellow') + " ..."
                self.percent_progress(10, self.MEMCACHED)
                return self.zonawibu(b)
        else:
            debug(self_get_netloc_req_url = self.get_netloc(req.url))
            debug(return_url = req.url)
            self.percent_progress(50, self.MEMCACHED)
            return req.url

    def upload_to_pcloud(self, url, download_path=os.getcwd(), name=None, username=None, password=None, folderid='0', foldername=None, downloadit=False, use_proxy=False):
        debug("upload_to_pcloud")
        from pyPCloud import pcloud
        if use_proxy:
            PCloud = pcloud(proxy=self.proxy)
        else:
            PCloud = pcloud()
        datax = PCloud.remoteUpload(url, username=username, password=password,folderid=folderid, renameit=name, foldername=foldername)
        idx = datax.get('metadata')[0].get('id')
        data, cookies = PCloud.getDownloadLink(idx, download_path=download_path)
        download_url = 'https://' + data.get('hosts')[0] + data.get('path')
        if downloadit:
            self.download(download_url, download_path)
        else:
            return download_url

    def pcloud(self, url_download, username = None, password = None, name = None, folderid = '0', foldername = None, downloadit = False, download_path = os.getcwd(), use_proxy=False):
        try:
            import pyPCloud
            return self.upload_to_pcloud(url_download, download_path, name, username, password, folderid, foldername, downloadit)
        except ImportError:
            PCLOUD_MODULE = ''
            if self.CONFIG.read_config('PCLOUD', 'PATH'):
                PARENT_PATH = self.CONFIG.read_config('PCLOUD', 'PATH')
                sys.path.append(PARENT_PATH)
                PCLOUD_MODULE = os.path.basename(PARENT_PATH)
                PARENT_PATH = os.path.dirname(PARENT_PATH)
                if configset.read_config('PCLOUD', 'NAME', 'aglink.ini'):
                    PCLOUD_MODULE = self.CONFIG.read_config('PCLOUD', 'NAME')
            if os.getenv('PCLOUD_MODULE'):
                PARENT_PATH = os.getenv('PCLOUD_MODULE')
                sys.path.append(PARENT_PATH)
                PCLOUD_MODULE = os.path.basename(PARENT_PATH)
                PARENT_PATH = os.path.dirname(PARENT_PATH)
            #else:
                #PARENT_PATH = os.path.join(os.path.dirname(os.path.dirname(__file__)),'test')
                #sys.path.append(os.path.join(PARENT_PATH, 'pypcloud'))
            debug(PARENT_PATH = PARENT_PATH)
            debug(PCLOUD_MODULE = PCLOUD_MODULE)
            sys.path.append(PARENT_PATH)
            try:
                import importlib
                pcloud = importlib.import_module(PCLOUD_MODULE)
                if use_proxy:
                    PCloud = pcloud.pcloud(proxy=self.proxy)
                else:
                    PCloud = pcloud.pcloud()
                datax = PCloud.remoteUpload(url_download, username = username, password = password, folderid = folderid, renameit = name, foldername = foldername)
                if downloadit:
                    idx = datax.get('metadata')[0].get('id')
                    data, cookies = PCloud.getDownloadLink(idx, download_path = download_path)
                    download_path = os.path.abspath(download_path)
                    if not os.path.isdir(download_path):
                        os.makedirs(download_path)
                    if not os.path.isdir(download_path):
                        download_path = os.path.dirname(__file__)
                    fileid = data.get('fileid')
                    #if name:
                        #PCloud.printlist('usage', renamefile='')
                        #PCloud.renameFile(name, fileid, None, username, password)
                    download_url = 'https://' + data.get('hosts')[0] + data.get('path')
                    if name:
                        return PCloud.download(download_url, download_path, name, cookies)
                    else:
                        return PCloud.download(download_url, download_path, cookies=cookies)
                return datax
            except:
                print traceback.format_exc()

    def test_proxy(self, proxies, timeout=3):
        if not proxies:
            return False
        try:
            a = requests.request('GET', self.url, proxies=proxies, verify=False, timeout=timeout)
            return True
        except:
            return False

    def setProxy(self, proxies=None, use_proxy=False):
        '''
            format proxies: ['http://ip:port', 'https://ip:port']
        '''
        debug()
        PROXY = {}
        if not self.CONFIG.read_config('PROXY', 'use_proxy'):
            if not use_proxy:
                return PROXY
        else:
            if proxies:  #data must list instance
                for i in proxies:
                    #host, port = str(i).split(":")
                    scheme = urlparse.urlparse(i).scheme
                    PROXY.update({scheme: i,})
                if not self.test_proxy(PROXY):
                    PROXY = {}
                    if self.CONFIG.read_config('PROXY', 'server') and self.CONFIG.read_config('PROXY', 'port'):
                        PROXY.update({
                        'http': 'http://%s:%s'%(self.CONFIG.read_config('PROXY', 'server'), self.CONFIG.read_config('PROXY', 'port')),
                        'https': 'https://%s:%s'%(self.CONFIG.read_config('PROXY', 'server'), self.CONFIG.read_config('PROXY', 'port'))
                        })
                # print "PROXY =", PROXY
                # print "self.test_proxy(PROXY) =", self.test_proxy(PROXY)
                if not self.test_proxy(PROXY):
                    from proxy_tester import proxy_tester
                    pt = proxy_tester.proxy_tester()
                    list_proxy_ok = pt.test_proxy_ip(self.url, print_list=True, limit=1)
                    debug(list_proxy_ok=list_proxy_ok)
                    if list_proxy_ok:
                        PROXY.update({
                            'http': 'http://%s'%(list_proxy_ok[0]),
                            'https': 'https://%s'%(list_proxy_ok[0])
                        })
            else:
                if use_proxy:
                    from proxy_tester import proxy_tester
                    pt = proxy_tester.proxy_tester()
                    list_proxy_ok = pt.test_proxy_ip(self.url, print_list=True, limit=1)
                    debug(list_proxy_ok=list_proxy_ok)
                    if list_proxy_ok:
                        PROXY.update({
                            'http': 'http://%s'%(list_proxy_ok[0]),
                            'https': 'https://%s'%(list_proxy_ok[0])
                        })
            if not self.proxy:
                self.proxy = PROXY
            if PROXY:
                for i in PROXY:
                    os.environ.update({i:PROXY.get(i)})
        return PROXY

    def set_percent_progress(self):
        while 1:
            percent = self.percent
            if not percent >= 100:
                if self.percent == percent:
                    if self.percent == 30:
                        self.percent += 30
                        self.percent_progress(self.percent, self.MEMCACHED)
                    else:
                        self.percent += 0.5
                        self.percent_progress(self.percent, self.MEMCACHED)
                    if percent >= 70:
                        time.sleep(0.5)
                    else:
                        time.sleep(1)
                else:
                    percent = self.percent
            else:
                self.percent_progress(percent, self.MEMCACHED)
                debug(percent = percent)
                break

    def daemon_generate(self, link, clip=None, quality=None, verbosity=None, support= False, direct_download=None, download_path=".", pcloud = False, pcloud_username = None, pcloud_password = None, pcloud_folderid = '0', pcloud_rename = None, pcloud_foldername = None, proxy = None, fast = False, bypass_regenerate = False, cliped = False, name = None, wget = False):
        args = (link, clip, quality, verbosity, support, direct_download, download_path, pcloud, pcloud_username, pcloud_password, pcloud_folderid, pcloud_rename, pcloud_foldername, proxy, fast, bypass_regenerate, cliped, name, wget)

        from multiprocessing.pool import ThreadPool
        pool = ThreadPool(processes = 2)
        t1 = pool.apply_async(self.generate, args)
        t2 = pool.apply_async(self.percent_progress, ())
        #t1 = threading.Thread(target = self.generate, args = args)
        #t2 = threading.Thread(target = self.set_percent_progress, args = ())
        #self.thread = t1
        mc_percent = int(self.get_percent_memcached())
        while 1:
            if not mc_percent == 100:
                ##print "mc_percent =", mc_percent
                mc_percent = int(self.get_percent_memcached())
                time.sleep(3)
            else:
                break
        link, name = t1.get()
        debug(link = link)
        debug(name = name)
        return link, name

    def usage(self):
        parser = argparse.ArgumentParser()
        parser.add_argument('LINK', help='Link to be convert', action='store', default = '')
        parser.add_argument(
            '-c', '--clip', help='Copy converted links to Clipboard', action='store_true', default = True)
        #parser.add_argument(
            #'-C', '--cliped', help='Genrate links from Clipboard', action='store_true', default = True)
        parser.add_argument(
            '-N', '--number', help='Number of Quality video download to', action='store')
        parser.add_argument(
            '-n', '--name', help='Name of video download to', action='store')
        parser.add_argument('-d', '--download',
                            help='Direct download', action='store_true')
        parser.add_argument(
            '-s', '--support', help='Show support text', action='store_true')
        parser.add_argument('-f', '--fast', help = 'Fast, no check', action = 'store_true')
        parser.add_argument(
            '-v', '--verbose', help='-v = version | -vv = Verbosity', action='count')
        parser.add_argument('-X', '--proxy', help='Set Proxy, format example http://127.0.0.1:8118 https://127.0.0.1:3128', action='store', nargs = '*')
        parser.add_argument('-x', '--use-proxy', help='Use Auto Proxy', action='store_true')
        parser.add_argument('-Xt', '--test-proxy', help='Test proxy with proxy option given', action='store_true')
        parser.add_argument(
            '-p', '--path', help='Download Path default this', action='store', default='.')
        parser.add_argument('-i', '--wget', action = 'store_true', help = 'Direct use wget (build in) for download manager')
        parser.add_argument('--pcloud', help = 'Remote Upload to Pcloud Storage', action = 'store_true')
        parser.add_argument('--pcloud-username', help = 'Username of Remote Upload to Pcloud Storage', action = 'store')
        parser.add_argument('--pcloud-password', help = 'Password of Remote Upload to Pcloud Storage', action = 'store')
        parser.add_argument('--pcloud-folderid', help = 'Folder ID Remote Upload to Pcloud Storage, default=0', action = 'store', default = '0', type = str)
        parser.add_argument('--pcloud-renameit', help = 'Rename After of Remote Upload to Pcloud Storage', action = 'store')
        parser.add_argument('--pcloud-foldername', help = 'Folder of Remote Upload to Pcloud Storage to', action = 'store')        
        parser.add_argument('-z', '--daemon', help = 'Generate with progress monitoring', action = 'store_true')        
        parser.add_argument('-m', '--memcached', help = 'Use progress monitoring with memcached ', action = 'store_true')        
        parser.add_argument('-ms', '--memcached-host', help = 'Progress monitoring with memcached server address/ip', action = 'store_true')        
        parser.add_argument('-mp', '--memcached-port', help = 'Progress monitoring with memcached server port', action = 'store_true')        

        if len(sys.argv) > 1:
            if '-v' == sys.argv[1]:
                print "version:", self.getVersion()
            else:
                args = parser.parse_args()
                if args.memcached:
                    self.MEMCACHED = True
                if args.memcached_host:
                    self.MEMCACHED_HOST = args.memcached_host
                if args.memcached_port:
                    self.MEMCACHED_PORT = args.memcached_port
                if args.proxy:
                    if self.CONFIG.read_config('PROXY', 'server') and self.CONFIG.read_config('PROXY', 'port'):
                        self.proxy.update(
                            {
                                'http': 'http://%s:%s'%(self.CONFIG.read_config('PROXY', 'server'), self.CONFIG.read_config('PROXY', 'port')), 
                                'https': 'https://%s:%s'%(self.CONFIG.read_config('PROXY', 'server'), self.CONFIG.read_config('PROXY', 'port'))
                            })
                    else:
                        self.setProxy(args.proxy)
                if args.proxy:
                    self.setProxy()

                if args.test_proxy:
                    return self.test_proxy()
                # print "ARGS - PATH =", args.path
                #print "args =", args
                cliped = False
                if args.LINK == 'c' or args.LINK == 'C':
                    cliped = True
                debug(args_link = args.LINK)
                if args.daemon:
                    self.daemon_generate(args.LINK, True, args.number,
                                         args.verbose, args.support, args.download, args.path, args.pcloud, args.pcloud_username, args.pcloud_password, args.pcloud_folderid, args.pcloud_renameit, args.pcloud_foldername, fast = args.fast, cliped= cliped, name = args.name, wget = args.wget)
                else:
                    self.generate(args.LINK, True, args.number,
                                  args.verbose, args.support, args.download, args.path, args.pcloud, args.pcloud_username, args.pcloud_password, args.pcloud_folderid, args.pcloud_renameit, args.pcloud_foldername, fast = args.fast, cliped= cliped, name = args.name, wget = args.wget, show_debug= args.verbose)
        else:
            parser.print_help()

if __name__ == '__main__':
    print "PID:", pid
    c = autogeneratelink()
    # c.mediafire('https://www.mediafire.com/file/dq0p6a9uj0sq4bl/')
    c.usage()
    # c.ourabeauty("http://ourabeauty.info/?id=R3R3ejRJOWNoQUQ4M2VkVUhRb2hodWNTcVJyQzMwUCt6cDI0V2FDZnhNWmlvNm56c1lzb016Tlh1UFBiemU3Vg==")
    #while 1:
        #percent = c.percent
        #if not percent == 100:
            #print "percent:", percent
            #if c.percent == percent:
                #if c.percent == 30:
                    #c.percent += 30
                #else:
                    #c.percent += 0.5
                #if percent >= 70:
                    #time.sleep(0.5)
                #else:
                    #time.sleep(1)
                #print "percent:", percent
            #else:
                #print "percent:", percent
                #percent = c.percent
        #else:
            #print "percent:", percent
            #debug(percent = percent)
            #break

    #url = 'https://www.youtube.com/watch?v=_s0nFWar9Co'
    #c.siotong(url, True)
    #print c.get_netloc(url)
    #c.youtube(url)
    #c.get_req(url)



#!/usr/bin/env python
"""
Download utility as an easy way to get file from the net

  python -m wget <URL>
  python wget.py <URL>

Downloads: http://pypi.python.org/pypi/wget/
Development: http://bitbucket.org/techtonik/python-wget/

wget.py is not option compatible with Unix wget utility,
to make command line interface intuitive for new people.

Public domain by anatoly techtonik <techtonik@gmail.com>
Also available under the terms of MIT license
Copyright (c) 2010-2015 anatoly techtonik
"""

__version__ = "3.1"


import sys, shutil, os
import tempfile
import math

PY3K = sys.version_info >= (3, 0)
if PY3K:
    import urllib.request as ulib
    import urllib.parse as urlparse
else:
    import urllib as ulib
    import urlparse


# --- workarounds for Python misbehavior ---

# enable passing unicode arguments from command line in Python 2.x
# https://stackoverflow.com/questions/846850/read-unicode-characters
def win32_utf8_argv():
    """Uses shell32.GetCommandLineArgvW to get sys.argv as a list of Unicode
    strings.

    Versions 2.x of Python don't support Unicode in sys.argv on
    Windows, with the underlying Windows API instead replacing multi-byte
    characters with '?'.
    """

    from ctypes import POINTER, byref, cdll, c_int, windll
    from ctypes.wintypes import LPCWSTR, LPWSTR

    GetCommandLineW = cdll.kernel32.GetCommandLineW
    GetCommandLineW.argtypes = []
    GetCommandLineW.restype = LPCWSTR

    CommandLineToArgvW = windll.shell32.CommandLineToArgvW
    CommandLineToArgvW.argtypes = [LPCWSTR, POINTER(c_int)]
    CommandLineToArgvW.restype = POINTER(LPWSTR)

    cmd = GetCommandLineW()
    argc = c_int(0)
    argv = CommandLineToArgvW(cmd, byref(argc))
    argnum = argc.value
    sysnum = len(sys.argv)
    result = []
    if argnum > 0:
        # Remove Python executable and commands if present
        start = argnum - sysnum
        for i in range(start, argnum):
            result.append(argv[i].encode('utf-8'))
    return result


# enable unicode output to windows console
# https://stackoverflow.com/questions/878972/windows-cmd-encoding-change-causes-python-crash
def win32_unicode_console():
    import codecs
    from ctypes import WINFUNCTYPE, windll, POINTER, byref, c_int
    from ctypes.wintypes import BOOL, HANDLE, DWORD, LPWSTR, LPCWSTR, LPVOID

    original_stderr = sys.stderr

    # Output exceptions in this code to original_stderr, so that we can at least see them
    def _complain(message):
        original_stderr.write(message if isinstance(message, str) else repr(message))
        original_stderr.write('\n')

    codecs.register(lambda name: codecs.lookup('utf-8') if name == 'cp65001' else None)

    try:
        GetStdHandle = WINFUNCTYPE(HANDLE, DWORD)(("GetStdHandle", windll.kernel32))
        STD_OUTPUT_HANDLE = DWORD(-11)
        STD_ERROR_HANDLE = DWORD(-12)
        GetFileType = WINFUNCTYPE(DWORD, DWORD)(("GetFileType", windll.kernel32))
        FILE_TYPE_CHAR = 0x0002
        FILE_TYPE_REMOTE = 0x8000
        GetConsoleMode = WINFUNCTYPE(BOOL, HANDLE, POINTER(DWORD))(("GetConsoleMode", windll.kernel32))
        INVALID_HANDLE_VALUE = DWORD(-1).value

        def not_a_console(handle):
            if handle == INVALID_HANDLE_VALUE or handle is None:
                return True
            return ((GetFileType(handle) & ~FILE_TYPE_REMOTE) != FILE_TYPE_CHAR
                    or GetConsoleMode(handle, byref(DWORD())) == 0)

        old_stdout_fileno = None
        old_stderr_fileno = None
        if hasattr(sys.stdout, 'fileno'):
            old_stdout_fileno = sys.stdout.fileno()
        if hasattr(sys.stderr, 'fileno'):
            old_stderr_fileno = sys.stderr.fileno()

        STDOUT_FILENO = 1
        STDERR_FILENO = 2
        real_stdout = (old_stdout_fileno == STDOUT_FILENO)
        real_stderr = (old_stderr_fileno == STDERR_FILENO)

        if real_stdout:
            hStdout = GetStdHandle(STD_OUTPUT_HANDLE)
            if not_a_console(hStdout):
                real_stdout = False

        if real_stderr:
            hStderr = GetStdHandle(STD_ERROR_HANDLE)
            if not_a_console(hStderr):
                real_stderr = False

        if real_stdout or real_stderr:
            WriteConsoleW = WINFUNCTYPE(BOOL, HANDLE, LPWSTR, DWORD, POINTER(DWORD), LPVOID)(("WriteConsoleW", windll.kernel32))

            class UnicodeOutput:
                def __init__(self, hConsole, stream, fileno, name):
                    self._hConsole = hConsole
                    self._stream = stream
                    self._fileno = fileno
                    self.closed = False
                    self.softspace = False
                    self.mode = 'w'
                    self.encoding = 'utf-8'
                    self.name = name
                    self.flush()

                def isatty(self):
                    return False

                def close(self):
                    # don't really close the handle, that would only cause problems
                    self.closed = True

                def fileno(self):
                    return self._fileno

                def flush(self):
                    if self._hConsole is None:
                        try:
                            self._stream.flush()
                        except Exception as e:
                            _complain("%s.flush: %r from %r" % (self.name, e, self._stream))
                            raise

                def write(self, text):
                    try:
                        if self._hConsole is None:
                            if not PY3K and isinstance(text, unicode):
                                text = text.encode('utf-8')
                            elif PY3K and isinstance(text, str):
                                text = text.encode('utf-8')
                            self._stream.write(text)
                        else:
                            if not PY3K and not isinstance(text, unicode):
                                text = str(text).decode('utf-8')
                            elif PY3K and not isinstance(text, str):
                                text = text.decode('utf-8')
                            remaining = len(text)
                            while remaining:
                                n = DWORD(0)
                                # There is a shorter-than-documented limitation on the
                                # length of the string passed to WriteConsoleW (see
                                # <http://tahoe-lafs.org/trac/tahoe-lafs/ticket/1232>.
                                retval = WriteConsoleW(self._hConsole, text, min(remaining, 10000), byref(n), None)
                                if retval == 0 or n.value == 0:
                                    raise IOError("WriteConsoleW returned %r, n.value = %r" % (retval, n.value))
                                remaining -= n.value
                                if not remaining:
                                    break
                                text = text[n.value:]
                    except Exception as e:
                        _complain("%s.write: %r" % (self.name, e))
                        raise

                def writelines(self, lines):
                    try:
                        for line in lines:
                            self.write(line)
                    except Exception as e:
                        _complain("%s.writelines: %r" % (self.name, e))
                        raise

            if real_stdout:
                sys.stdout = UnicodeOutput(hStdout, None, STDOUT_FILENO, '<Unicode console stdout>')
            else:
                sys.stdout = UnicodeOutput(None, sys.stdout, old_stdout_fileno, '<Unicode redirected stdout>')

            if real_stderr:
                sys.stderr = UnicodeOutput(hStderr, None, STDERR_FILENO, '<Unicode console stderr>')
            else:
                sys.stderr = UnicodeOutput(None, sys.stderr, old_stderr_fileno, '<Unicode redirected stderr>')
    except Exception as e:
        _complain("exception %r while fixing up sys.stdout and sys.stderr" % (e,))


# --- helpers ---

def to_unicode(filename):
    """:return: filename decoded from utf-8 to unicode"""
    #
    if PY3K:
        # [ ] test this on Python 3 + (Windows, Linux)
        # [ ] port filename_from_headers once this works
        # [ ] add test to repository / Travis
        return filename
    else:
        return unicode(filename, 'utf-8')

def filename_from_url(url):
    """:return: detected filename as unicode or None"""
    fname = os.path.basename(urlparse.urlparse(url).path)
    if len(fname.strip(" \n\t.")) == 0:
        return None
    return to_unicode(fname)

def filename_from_headers(headers):
    """Detect filename from Content-Disposition headers if present.
    http://greenbytes.de/tech/tc2231/

    :param: headers as dict, list or string
    :return: filename from content-disposition header or None
    """
    if type(headers) == str:
        headers = headers.splitlines()
    if type(headers) == list:
        headers = dict([x.split(':', 1) for x in headers])
    cdisp = headers.get("Content-Disposition")
    if not cdisp:
        return None
    cdtype = cdisp.split(';')
    if len(cdtype) == 1:
        return None
    if cdtype[0].strip().lower() not in ('inline', 'attachment'):
        return None
    # several filename params is illegal, but just in case
    fnames = [x for x in cdtype[1:] if x.strip().startswith('filename=')]
    if len(fnames) > 1:
        return None
    name = fnames[0].split('=')[1].strip(' \t"')
    name = os.path.basename(name)
    if not name:
        return None
    return name

def filename_fix_existing(filename):
    """Expands name portion of filename with numeric ' (x)' suffix to
    return filename that doesn't exist already.
    """
    dirname = u'.'
    name, ext = filename.rsplit('.', 1)
    names = [x for x in os.listdir(dirname) if x.startswith(name)]
    names = [x.rsplit('.', 1)[0] for x in names]
    suffixes = [x.replace(name, '') for x in names]
    # filter suffixes that match ' (x)' pattern
    suffixes = [x[2:-1] for x in suffixes
                if x.startswith(' (') and x.endswith(')')]
    indexes  = [int(x) for x in suffixes
                if set(x) <= set('0123456789')]
    idx = 1
    if indexes:
        idx += sorted(indexes)[-1]
    return '%s (%d).%s' % (name, idx, ext)


# --- terminal/console output helpers ---

def get_console_width():
    """Return width of available window area. Autodetection works for
       Windows and POSIX platforms. Returns 80 for others

       Code from http://bitbucket.org/techtonik/python-pager
    """

    if os.name == 'nt':
        STD_INPUT_HANDLE  = -10
        STD_OUTPUT_HANDLE = -11
        STD_ERROR_HANDLE  = -12

        # get console handle
        from ctypes import windll, Structure, byref
        try:
            from ctypes.wintypes import SHORT, WORD, DWORD
        except ImportError:
            # workaround for missing types in Python 2.5
            from ctypes import (
                c_short as SHORT, c_ushort as WORD, c_ulong as DWORD)
        console_handle = windll.kernel32.GetStdHandle(STD_OUTPUT_HANDLE)

        # CONSOLE_SCREEN_BUFFER_INFO Structure
        class COORD(Structure):
            _fields_ = [("X", SHORT), ("Y", SHORT)]

        class SMALL_RECT(Structure):
            _fields_ = [("Left", SHORT), ("Top", SHORT),
                        ("Right", SHORT), ("Bottom", SHORT)]

        class CONSOLE_SCREEN_BUFFER_INFO(Structure):
            _fields_ = [("dwSize", COORD),
                        ("dwCursorPosition", COORD),
                        ("wAttributes", WORD),
                        ("srWindow", SMALL_RECT),
                        ("dwMaximumWindowSize", DWORD)]

        sbi = CONSOLE_SCREEN_BUFFER_INFO()
        ret = windll.kernel32.GetConsoleScreenBufferInfo(
            console_handle, byref(sbi))
        if ret == 0:
            return 0
        return sbi.srWindow.Right+1

    elif os.name == 'posix':
        from fcntl import ioctl
        from termios import TIOCGWINSZ
        from array import array

        winsize = array("H", [0] * 4)
        try:
            ioctl(sys.stdout.fileno(), TIOCGWINSZ, winsize)
        except IOError:
            pass
        return (winsize[1], winsize[0])[0]

    return 80


def bar_thermometer(current, total, width=80):
    """Return thermometer style progress bar string. `total` argument
    can not be zero. The minimum size of bar returned is 3. Example:

        [..........            ]

    Control and trailing symbols (\r and spaces) are not included.
    See `bar_adaptive` for more information.
    """
    # number of dots on thermometer scale
    avail_dots = width-2
    shaded_dots = int(math.floor(float(current) / total * avail_dots))
    return '[' + '.'*shaded_dots + ' '*(avail_dots-shaded_dots) + ']'

def bar_adaptive(current, total, width=80):
    """Return progress bar string for given values in one of three
    styles depending on available width:

        [..  ] downloaded / total
        downloaded / total
        [.. ]

    if total value is unknown or <= 0, show bytes counter using two
    adaptive styles:

        %s / unknown
        %s

    if there is not enough space on the screen, do not display anything

    returned string doesn't include control characters like \r used to
    place cursor at the beginning of the line to erase previous content.

    this function leaves one free character at the end of string to
    avoid automatic linefeed on Windows.
    """

    # process special case when total size is unknown and return immediately
    if not total or total < 0:
        msg = "%s / unknown" % current
        if len(msg) < width:    # leaves one character to avoid linefeed
            return msg
        if len("%s" % current) < width:
            return "%s" % current

    # --- adaptive layout algorithm ---
    #
    # [x] describe the format of the progress bar
    # [x] describe min width for each data field
    # [x] set priorities for each element
    # [x] select elements to be shown
    #   [x] choose top priority element min_width < avail_width
    #   [x] lessen avail_width by value if min_width
    #   [x] exclude element from priority list and repeat

    #  10% [.. ]  10/100
    # pppp bbbbb sssssss

    min_width = {
        'percent': 4,  # 100%
        'bar': 3,      # [.]
        'size': len("%s" % total)*2 + 3, # 'xxxx / yyyy'
    }
    priority = ['percent', 'bar', 'size']

    # select elements to show
    selected = []
    avail = width
    for field in priority:
        if min_width[field] < avail:
            selected.append(field)
            avail -= min_width[field]+1   # +1 is for separator or for reserved space at
                                                                        # the end of line to avoid linefeed on Windows
    # render
    output = ''
    for field in selected:

        if field == 'percent':
            # fixed size width for percentage
            output += ('%s%%' % (100 * current // total)).rjust(min_width['percent'])
        elif field == 'bar':  # [. ]
            # bar takes its min width + all available space
            output += bar_thermometer(current, total, min_width['bar']+avail)
        elif field == 'size':
            # size field has a constant width (min == max)
            output += ("%s / %s" % (current, total)).rjust(min_width['size'])

        selected = selected[1:]
        if selected:
            output += ' '  # add field separator

    return output

# --/ console helpers


__current_size = 0  # global state variable, which exists solely as a
                                        # workaround against Python 3.3.0 regression
                                        # http://bugs.python.org/issue16409
                                        # fixed in Python 3.3.1
def callback_progress(blocks, block_size, total_size, bar_function):
    """callback function for urlretrieve that is called when connection is
    created and when once for each block

    draws adaptive progress bar in terminal/console

    use sys.stdout.write() instead of "print,", because it allows one more
    symbol at the line end without linefeed on Windows

    :param blocks: number of blocks transferred so far
    :param block_size: in bytes
    :param total_size: in bytes, can be -1 if server doesn't return it
    :param bar_function: another callback function to visualize progress
    """
    global __current_size

    width = min(100, get_console_width())

    if sys.version_info[:3] == (3, 3, 0):  # regression workaround
        if blocks == 0:  # first call
            __current_size = 0
        else:
            __current_size += block_size
        current_size = __current_size
    else:
        current_size = min(blocks*block_size, total_size)
    progress = bar_function(current_size, total_size, width)
    if progress:
        sys.stdout.write("\r" + progress)

def valid_name(name):
    import string
    valid_chars = "-_.() %s%s" % (string.ascii_letters, string.digits)
    return ''.join(c for c in name if c in valid_chars)


def detect_filename(url=None, out=None, headers=None, default="download.wget"):
    """Return filename for saving file. If no filename is detected from output
    argument, url or headers, return default (download.wget)
    """
    names = dict(out='', url='', headers='')
    if out:
        names["out"] = out or ''
    if url:
        names["url"] = filename_from_url(url) or ''
    if headers:
        names["headers"] = filename_from_headers(headers) or ''
    return names["out"] or names["headers"] or names["url"] or default

def download(url, out=".", bar=bar_adaptive):
    """High level function, which downloads URL into tmp file in current
    directory and then renames it to filename autodetected from either URL
    or HTTP headers.

    :param bar: function to track download progress (visualize etc.)
    :param out: output filename or directory
    :return:    filename where URL is downloaded to
    """

    # detect of out is a directory
    outdir = None
    if out and os.path.isdir(out):
        outdir = out
        out = None
        
    # get filename for temp file in current directory
    prefix = detect_filename(url, out)
    #(fd, tmpfile) = tempfile.mkstemp(".tmp", prefix=prefix, dir=".")
    (fd, tmpfile) = tempfile.mkstemp(".tmp", prefix=prefix, dir= outdir)
    os.close(fd)
    # os.unlink(tmpfile)

    # set progress monitoring callback
    def callback_charged(blocks, block_size, total_size):
        # 'closure' to set bar drawing function in callback
        callback_progress(blocks, block_size, total_size, bar_function=bar)
    if bar:
        callback = callback_charged
    else:
        callback = None

    if PY3K:
        # Python 3 can not quote URL as needed
        binurl = list(urlparse.urlsplit(url))
        binurl[2] = urlparse.quote(binurl[2])
        binurl = urlparse.urlunsplit(binurl)
    else:
        binurl = url
    (tmpfile, headers) = ulib.urlretrieve(binurl, tmpfile, callback)
    filename = detect_filename(url, out, headers)
    if outdir:
        #filename = outdir + "/" + filename
        filename = os.path.join(outdir, filename)
    # add numeric ' (x)' suffix if filename already exists
    if os.path.exists(filename):
        filename = filename_fix_existing(filename)
    root_dir, basename = os.path.split(filename)
    basename = valid_name(basename)
    filename = os.path.join(root_dir, basename)
    
    shutil.move(tmpfile, filename)

    #print headers
    # os.unlink(tmpfile)
    return filename


usage = """\
usage: wget.py [options] URL

options:
  -o --output FILE|DIR   output filename or directory
  -h --help
  --version
"""

if __name__ == "__main__":
    if len(sys.argv) < 2 or "-h" in sys.argv or "--help" in sys.argv:
        sys.exit(usage)
    if "--version" in sys.argv:
        sys.exit("wget.py " + __version__)

    # patch Python 2.x to read unicode from command line
    if not PY3K and sys.platform == "win32":
        sys.argv = win32_utf8_argv()
    # patch Python to write unicode characters to console
    if sys.platform == "win32":
        win32_unicode_console()

    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("-o", "--output", dest="output")
    (options, args) = parser.parse_args()

    url = sys.argv[1]
    filename = download(args[0], out=options.output)

    print("")
    print("Saved under %s" % os.path.abspath(filename))


r"""
features that require more tuits for urlretrieve API
http://www.python.org/doc/2.6/library/urllib.html#urllib.urlretrieve

[x] autodetect filename from URL
[x] autodetect filename from headers - Content-Disposition
    http://greenbytes.de/tech/tc2231/
[ ] make HEAD request to detect temp filename from Content-Disposition
[ ] process HTTP status codes (i.e. 404 error)
    http://ftp.de.debian.org/debian/pool/iso-codes_3.24.2.orig.tar.bz2
[ ] catch KeyboardInterrupt
[ ] optionally preserve incomplete file
[x] create temp file in current directory
[ ] resume download (broken connection)
[ ] resume download (incomplete file)
[x] show progress indicator
    http://mail.python.org/pipermail/tutor/2005-May/038797.html
[x] do not overwrite downloaded file
 [x] rename file automatically if exists
[x] optionally specify path for downloaded file

[ ] options plan
 [x] -h, --help, --version (CHAOS speccy)
[ ] clpbar progress bar style
_ 30.0Mb at  3.0 Mbps  eta:   0:00:20   30% [=====         ]
[ ] test "bar \r" print with \r at the end of line on Windows
[ ] process Python 2.x urllib.ContentTooShortError exception gracefully
    (ideally retry and continue download)

    (tmpfile, headers) = urllib.urlretrieve(url, tmpfile, callback_progress)
  File "C:\Python27\lib\urllib.py", line 93, in urlretrieve
    return _urlopener.retrieve(url, filename, reporthook, data)
  File "C:\Python27\lib\urllib.py", line 283, in retrieve
    "of %i bytes" % (read, size), result)
urllib.ContentTooShortError: retrieval incomplete: got only 15239952 out of 24807571 bytes

[ ] find out if urlretrieve may return unicode headers
[ ] write files with unicode characters
    https://bitbucket.org/techtonik/python-wget/issues/7/filename-issue
  [x] Python 2, Windows
  [x] Python 3, Windows
  [ ] Linux
[ ] add automatic tests
  [ ] specify unicode URL from command line
  [ ] specify unicode output file from command line
  [ ] test suite for unsafe filenames from url and from headers

[ ] security checks
  [ ] filename_from_url
  [ ] filename_from_headers
  [ ] MITM redirect from https URL
  [ ] https certificate check
  [ ] size+hash check helpers
    [ ] fail if size is known and mismatch
    [ ] fail if hash mismatch
"""



#!/usr/bin/python

"""
    Other Repositories of python-ping
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    * https://github.com/l4m3rx/python-ping      supports Python2 and Python3
    * https://bitbucket.org/delroth/python-ping


    About
    ~~~~~

    A pure python ping implementation using raw socket.


    Note that ICMP messages can only be sent from processes running as root.


    Derived from ping.c distributed in Linux's netkit. That code is
    copyright (c) 1989 by The Regents of the University of California.
    That code is in turn derived from code written by Mike Muuss of the
    US Army Ballistic Research Laboratory in December, 1983 and
    placed in the public domain. They have my thanks.

    Bugs are naturally mine. I'd be glad to hear about them. There are
    certainly word - size dependenceies here.

    Copyright (c) Matthew Dixon Cowles, <http://www.visi.com/~mdc/>.
    Distributable under the terms of the GNU General Public License
    version 2. Provided with no warranties of any sort.

    Original Version from Matthew Dixon Cowles:
      -> ftp://ftp.visi.com/users/mdc/ping.py

    Rewrite by Jens Diemer:
      -> http://www.python-forum.de/post-69122.html#69122

    Rewrite by Johannes Meyer:
      -> http://www.python-forum.de/viewtopic.php?p=183720


    Revision history
    ~~~~~~~~~~~~~~~~

    November 1, 2010
    Rewrite by Johannes Meyer:
     -  changed entire code layout
     -  changed some comments and docstrings
     -  replaced time.clock() with time.time() in order
        to be able to use this module on linux, too.
     -  added global __all__, ICMP_CODE and ERROR_DESCR
     -  merged functions "do_one" and "send_one_ping"
     -  placed icmp packet creation in its own function
     -  removed timestamp from the icmp packet
     -  added function "multi_ping_query"
     -  added class "PingQuery"

    May 30, 2007
    little rewrite by Jens Diemer:
     -  change socket asterisk import to a normal import
     -  replace time.time() with time.clock()
     -  delete "return None" (or change to "return" only)
     -  in checksum() rename "str" to "source_string"

    November 22, 1997
    Initial hack. Doesn't do much, but rather than try to guess
    what features I (or others) will want in the future, I've only
    put in what I need now.

    December 16, 1997
    For some reason, the checksum bytes are in the wrong order when
    this is run under Solaris 2.X for SPARC but it works right under
    Linux x86. Since I don't know just what's wrong, I'll swap the
    bytes always and then do an htons().

    December 4, 2000
    Changed the struct.pack() calls to pack the checksum and ID as
    unsigned. My thanks to Jerome Poincheval for the fix.

"""

import time
import socket
import struct
import select
import random
import asyncore

# From /usr/include/linux/icmp.h; your milage may vary.
ICMP_ECHO_REQUEST = 8 # Seems to be the same on Solaris.

ICMP_CODE = socket.getprotobyname('icmp')
ERROR_DESCR = {
    1: ' - Note that ICMP messages can only be '
       'sent from processes running as root.',
    10013: ' - Note that ICMP messages can only be sent by'
           ' users or processes with administrator rights.'
    }

__all__ = ['create_packet', 'do_one', 'verbose_ping', 'PingQuery',
           'multi_ping_query']


def checksum(source_string):
    # I'm not too confident that this is right but testing seems to
    # suggest that it gives the same answers as in_cksum in ping.c.
    sum = 0
    count_to = (len(source_string) / 2) * 2
    count = 0
    while count < count_to:
        this_val = ord(source_string[count + 1])*256+ord(source_string[count])
        sum = sum + this_val
        sum = sum & 0xffffffff # Necessary?
        count = count + 2
    if count_to < len(source_string):
        sum = sum + ord(source_string[len(source_string) - 1])
        sum = sum & 0xffffffff # Necessary?
    sum = (sum >> 16) + (sum & 0xffff)
    sum = sum + (sum >> 16)
    answer = ~sum
    answer = answer & 0xffff
    # Swap bytes. Bugger me if I know why.
    answer = answer >> 8 | (answer << 8 & 0xff00)
    return answer


def create_packet(id):
    """Create a new echo request packet based on the given "id"."""
    # Header is type (8), code (8), checksum (16), id (16), sequence (16)
    header = struct.pack('bbHHh', ICMP_ECHO_REQUEST, 0, 0, id, 1)
    data = 192 * 'Q'
    # Calculate the checksum on the data and the dummy header.
    my_checksum = checksum(header + data)
    # Now that we have the right checksum, we put that in. It's just easier
    # to make up a new header than to stuff it into the dummy.
    header = struct.pack('bbHHh', ICMP_ECHO_REQUEST, 0,
                         socket.htons(my_checksum), id, 1)
    return header + data


def do_one(dest_addr, timeout=1):
    """
    Sends one ping to the given "dest_addr" which can be an ip or hostname.
    "timeout" can be any integer or float except negatives and zero.

    Returns either the delay (in seconds) or None on timeout and an invalid
    address, respectively.

    """
    try:
        my_socket = socket.socket(socket.AF_INET, socket.SOCK_RAW, ICMP_CODE)
    except socket.error as e:
        if e.errno in ERROR_DESCR:
            # Operation not permitted
            raise socket.error(''.join((e.args[1], ERROR_DESCR[e.errno])))
        raise # raise the original error
    try:
        host = socket.gethostbyname(dest_addr)
    except socket.gaierror:
        return
    # Maximum for an unsigned short int c object counts to 65535 so
    # we have to sure that our packet id is not greater than that.
    packet_id = int((id(timeout) * random.random()) % 65535)
    packet = create_packet(packet_id)
    while packet:
        # The icmp protocol does not use a port, but the function
        # below expects it, so we just give it a dummy port.
        sent = my_socket.sendto(packet, (dest_addr, 1))
        packet = packet[sent:]
    delay = receive_ping(my_socket, packet_id, time.time(), timeout)
    my_socket.close()
    return delay


def receive_ping(my_socket, packet_id, time_sent, timeout):
    # Receive the ping from the socket.
    time_left = timeout
    while True:
        started_select = time.time()
        ready = select.select([my_socket], [], [], time_left)
        how_long_in_select = time.time() - started_select
        if ready[0] == []: # Timeout
            return
        time_received = time.time()
        rec_packet, addr = my_socket.recvfrom(1024)
        icmp_header = rec_packet[20:28]
        type, code, checksum, p_id, sequence = struct.unpack(
            'bbHHh', icmp_header)
        if p_id == packet_id:
            return time_received - time_sent
        time_left -= time_received - time_sent
        if time_left <= 0:
            return


def verbose_ping(dest_addr, timeout=2, count=4):
    """
    Sends one ping to the given "dest_addr" which can be an ip or hostname.

    "timeout" can be any integer or float except negatives and zero.
    "count" specifies how many pings will be sent.

    Displays the result on the screen.
 
    """
    for i in range(count):
        print('ping {}...'.format(dest_addr))
        delay = do_one(dest_addr, timeout)
        if delay == None:
            print('failed. (Timeout within {} seconds.)'.format(timeout))
        else:
            delay = round(delay * 1000.0, 4)
            print('get ping in {} milliseconds.'.format(delay))
    print('')
    
def vping(dest_addr, timeout=2, count=4):
    """
    Sends one ping to the given "dest_addr" which can be an ip or hostname.

    "timeout" can be any integer or float except negatives and zero.
    "count" specifies how many pings will be sent.

    Displays the result on the screen.
 
    """
    for i in range(count):
        #print('ping {}...'.format(dest_addr))
        delay = do_one(dest_addr, timeout)
        if delay == None:
            return False
        else:
            delay = round(delay * 1000.0, 4)
            #print('get ping in {} milliseconds.'.format(delay))
            return True
    print('')


class PingQuery(asyncore.dispatcher):
    def __init__(self, host, p_id, timeout=0.5, ignore_errors=False):
        """
       Derived class from "asyncore.dispatcher" for sending and
       receiving an icmp echo request/reply.
       
       Usually this class is used in conjunction with the "loop"
       function of asyncore.
       
       Once the loop is over, you can retrieve the results with
       the "get_result" method. Assignment is possible through
       the "get_host" method.
       
       "host" represents the address under which the server can be reached.
       "timeout" is the interval which the host gets granted for its reply.
       "p_id" must be any unique integer or float except negatives and zeros.
       
       If "ignore_errors" is True, the default behaviour of asyncore
       will be overwritten with a function which does just nothing.
       
       """
        asyncore.dispatcher.__init__(self)
        try:
            self.create_socket(socket.AF_INET, socket.SOCK_RAW, ICMP_CODE)
        except socket.error as e:
            if e.errno in ERROR_DESCR:
                # Operation not permitted
                raise socket.error(''.join((e.args[1], ERROR_DESCR[e.errno])))
            raise # raise the original error
        self.time_received = 0
        self.time_sent = 0
        self.timeout = timeout
        # Maximum for an unsigned short int c object counts to 65535 so
        # we have to sure that our packet id is not greater than that.
        self.packet_id = int((id(timeout) / p_id) % 65535)
        self.host = host
        self.packet = create_packet(self.packet_id)
        if ignore_errors:
            # If it does not care whether an error occured or not.
            self.handle_error = self.do_not_handle_errors
            self.handle_expt = self.do_not_handle_errors

    def writable(self):
        return self.time_sent == 0

    def handle_write(self):
        self.time_sent = time.time()
        while self.packet:
            # The icmp protocol does not use a port, but the function
            # below expects it, so we just give it a dummy port.
            sent = self.sendto(self.packet, (self.host, 1))
            self.packet = self.packet[sent:]

    def readable(self):
        # As long as we did not sent anything, the channel has to be left open.
        if (not self.writable()
            # Once we sent something, we should periodically check if the reply
            # timed out.
            and self.timeout < (time.time() - self.time_sent)):
            self.close()
            return False
        # If the channel should not be closed, we do not want to read something
        # until we did not sent anything.
        return not self.writable()

    def handle_read(self):
        read_time = time.time()
        packet, addr = self.recvfrom(1024)
        header = packet[20:28]
        type, code, checksum, p_id, sequence = struct.unpack("bbHHh", header)
        if p_id == self.packet_id:
            # This comparison is necessary because winsocks do not only get
            # the replies for their own sent packets.
            self.time_received = read_time
            self.close()

    def get_result(self):
        """Return the ping delay if possible, otherwise None."""
        if self.time_received > 0:
            return self.time_received - self.time_sent

    def get_host(self):
        """Return the host where to the request has or should been sent."""
        return self.host

    def do_not_handle_errors(self):
        # Just a dummy handler to stop traceback printing, if desired.
        pass

    def create_socket(self, family, type, proto):
        # Overwritten, because the original does not support the "proto" arg.
        sock = socket.socket(family, type, proto)
        sock.setblocking(0)
        self.set_socket(sock)
        # Part of the original but is not used. (at least at python 2.7)
        # Copied for possible compatiblity reasons.
        self.family_and_type = family, type

    # If the following methods would not be there, we would see some very
    # "useful" warnings from asyncore, maybe. But we do not want to, or do we?
    def handle_connect(self):
        pass

    def handle_accept(self):
        pass

    def handle_close(self):
        self.close()


def multi_ping_query(hosts, timeout=1, step=512, ignore_errors=False):
    """
    Sends multiple icmp echo requests at once.

    "hosts" is a list of ips or hostnames which should be pinged.
    "timeout" must be given and a integer or float greater than zero.
    "step" is the amount of sockets which should be watched at once.

    See the docstring of "PingQuery" for the meaning of "ignore_erros".

    """
    results, host_list, id = {}, [], 0
    for host in hosts:
        try:
            host_list.append(socket.gethostbyname(host))
        except socket.gaierror:
            results[host] = None
    while host_list:
        sock_list = []
        for ip in host_list[:step]: # select supports only a max of 512
            id += 1
            sock_list.append(PingQuery(ip, id, timeout, ignore_errors))
            host_list.remove(ip)
        # Remember to use a timeout here. The risk to get an infinite loop
        # is high, because noone can guarantee that each host will reply!
        asyncore.loop(timeout)
        for sock in sock_list:
            results[sock.get_host()] = sock.get_result()
    return results


if __name__ == '__main__':
    # Testing
    verbose_ping('www.heise.de')
    verbose_ping('google.com')
    verbose_ping('an-invalid-test-url.com')
    verbose_ping('127.0.0.1')
    host_list = ['www.heise.de', 'google.com', '127.0.0.1',
                 'an-invalid-test-url.com']
    for host, ping in multi_ping_query(host_list).iteritems():
        print(host, '=', ping)



import sys
import comtypes.client as cc
import comtypes
import argparse
import os
import argparse

class IDMan(object):
    def __init__(self):
        super(IDMan, self)
        self.tlb = r'c:\Program Files\Internet Download Manager\idmantypeinfo.tlb'
        if not self.tlb:
            self.tlb = r'c:\Program Files\Internet Download Manager (x86)\idmantypeinfo.tlb'
        if not self.tlb:
            print "It seem IDM not installer, please install first !"
            sys.exit("It seem IDM not installer, please install first !")

    def get_from_clipboard(self):
        try:
            import clipboard
        except ImportError:
            print "Module Clipboard not Installer yet, please install first"
            q = raw_input("Please re-input url download to:")
            if not q:
                sys.exit("You not input URL Download !")
            else:
                return q
        return clipboard.paste()

    def download(self, link, path_to_save=None, output=None, referrer=None, cookie=None, postData=None, user=None, password=None, confirm = False, lflag = None, clip=False):
        #print "link =", link
        #print "referrer =", referrer
        #print "postData =", postData
        #print "user     =", user
        #print "password =", password
        #print "path_to_save =", path_to_save
        #print "output   =", output
        #print "lflag    =", lflag

        if clip:
            link = self.get_from_clipboard()
        if confirm:
            lflag = 0
        else:
            lflag = 5
        try:
            cc.GetModule(['{ECF21EAB-3AA8-4355-82BE-F777990001DD}', 1, 0])
        except:
            cc.GetModule(self.tlb)

        import comtypes.gen.IDManLib as idman 
        idman1 = cc.CreateObject(idman.CIDMLinkTransmitter, None, None, idman.ICIDMLinkTransmitter2)
        if path_to_save:
            os.path.realpath(path_to_save)
        idman1.SendLinkToIDM(link, referrer, cookie, postData, user, password, path_to_save, output, lflag)
    
    def usage(self):
        parse = argparse.ArgumentParser(formatter_class=argparse.RawTextHelpFormatter)
        parse.add_argument('URL', action='store', help='url to download')
        parse.add_argument('-p', '--path', action='store', help='Path to save', default=os.getcwd())
        parse.add_argument('-o', '--output', help='Save with different name', action='store')
        parse.add_argument('-c', '--confirm', help='Confirm before download', action='store_true')
        parse.add_argument('-C', '--clip', help='Get URL from clipboard', action='store_true')
        if len(sys.argv) ==1:
            parse.print_help()
        else:
            args = parse.parse_args()
            self.download(args.URL, args.path, args.output, confirm=args.confirm, clip=args.clip)
        

if __name__ == '__main__':
    c = IDMan()
    c.usage()


from __future__ import print_function
import sys
import argparse
if sys.version_info.major == 2:
    import ConfigParser
else:
    import configparser as ConfigParser
import os
import traceback
import re
from collections import OrderedDict
import inspect
#from debug import debug 
# from make_colors import make_colors

__sdk__ = '2.7'
__platform__ = 'all'
__url__ = 'licface@yahoo.com'
__build__ = '2.7'

configname ='conf.ini'
PATH = ''
if PATH:
    configname = os.path.join(PATH, os.path.basename(configname))

class MultiOrderedDict(OrderedDict):
    def __setitem__(self, key, value):
        if isinstance(value, list) and key in self:
            self[key].extend(value)
        else:
            super(OrderedDict, self).__setitem__(key, value)


class configset(object):
    cfg = ConfigParser.RawConfigParser(allow_no_value=True)
    cfg.optionxform = str
    THIS_PATH = os.path.dirname(__file__)
    # configname ='conf.ini'
    # #debug(configname = configname)
    
    def __init__(self):
        super(configset, self)
        global configname
        global PATH
        self.cfg = ConfigParser.RawConfigParser(allow_no_value=True)
        self.cfg.optionxform = str        
        self.configname = configname
        if self.configname:
            configname = self.configname

        self.path = None
        if not self.path:
            self.path = os.path.dirname(inspect.stack()[0][1])
        if PATH:
            self.path = PATH
        # debug(self_path = self.path)
        
        configname = os.path.join(self.path, os.path.basename(configname))
        # debug(configname=configname)
    
    def get_config_file(self, filename='', verbosity=None):
        if not filename:
            filename = self.configname
        configname = filename
        self.configname = configname
        #debug(configname = filename)
        self.configname = configname
        #debug(configset_configname = self.configname)
        self.path = None
        if self.path:
            if os.getenv('DEBUG'):
                print ("001")
            if configname:
                self.configname = os.path.join(os.path.abspath(self.path), os.path.basename(self.configname))
        
        if os.path.isfile(os.path.join(os.getcwd(), filename)):
            if os.getenv('DEBUG'):
                print ("002")
            #debug(checking_001 = "os.path.isfile(os.path.join(os.getcwd(), filename))")
            self.configname = os.path.join(os.getcwd(), filename)
            #debug(configname = os.path.join(os.getcwd(), filename))
            return os.path.join(os.getcwd(), filename)
        elif os.path.isfile(filename):
            if os.getenv('DEBUG'):
                print ("003")
            #debug(checking_002 = "os.path.isfile(filename)")
            self.configname =filename
            #debug(configname = os.path.abspath(filename))
            return filename
        elif os.path.isfile(os.path.join(os.path.dirname(__file__), filename)):
            if os.getenv('DEBUG'):
                print ("004")
            #debug(checking_003 = "os.path.isfile(os.path.join(os.path.dirname(__file__), filename))")
            self.configname =os.path.join(os.path.dirname(__file__), filename)
            #debug(configname = os.path.join(os.path.dirname(__file__), filename))
            return os.path.join(os.path.dirname(__file__), filename)
        elif os.path.isfile(self.configname):
            if os.getenv('DEBUG'):
                print ("005")
            #debug(checking_004 = "os.path.isfile(configname)")
            #debug(configname = os.path.abspath(configname))
            return configname
        else:
            if os.getenv('DEBUG'):
                print ("006")
            #debug(checking_006 = "ELSE")
            fcfg = self.configname
            f = open(fcfg, 'w')
            f.close()
            filecfg = fcfg
            #debug(CREATE = os.path.abspath(filecfg))
            return filecfg
    
    def write_config(self, section, option, filename='', value=None, cfg = None, verbosity=None):
        #print ("SECTION:", section)
        #print ("OPTION :", option)
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        if not cfg:
            cfg = configset.cfg
        if cfg:
            cfg.read(filename)
        else:
            cfg = ConfigParser.RawConfigParser(allow_no_value=True)
            cfg.optionxform = str
            cfg.read(filename)
        try:
            cfg.set(section, option, value)
        except ConfigParser.NoSectionError:
            cfg.add_section(section)
            cfg.set(section, option, value)
        except ConfigParser.NoOptionError:
            cfg.set(section, option, value)

        if os.path.isfile(filename):
            cfg_data = open(filename,'w+')
        else:
            cfg_data = open(filename,'wb')

        cfg.write(cfg_data) 
        cfg_data.close()  
        
        return self.read_config(section, option, filename)
        
    def write_config2(self, section, option, filename='', value=None, verbosity=None):
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        #cfg = ConfigParser.RawConfigParser(allow_no_value=True, dict_type=MultiOrderedDict) 
        if not value == None:
            if os.path.isfile(os.path.join(THIS_PATH, filename)):
                configset.cfg.read(filename)
            else:
                filename = self.get_config_file()
                configset.cfg.read(filename)
            try:
                configset.cfg.get(section, option)
                configset.cfg.set(section, option, value)
            except ConfigParser.NoSectionError:
                #configset.cfg.add_section(section)
                #configset.cfg.set(section, option, value)
                return "\tNo Section Name: '%s'" %(section)
            except ConfigParser.NoOptionError:
                return "\tNo Option Name: '%s'" %(option)
            cfg_data = open(filename,'wb')
            configset.cfg.write(cfg_data)   
            cfg_data.close()
            return ''
        else:
            return "No Value set !"
    
    def read_config(self, section, option, filename='', value=None, verbosity=None):
        """
            option: section, option, filename='', value=None
        """
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        #print ("FILENAME =", filename)
        self.cfg.read(filename)
        #print ("SECTION =", section)
        #print ("OPTION  =", option)        
        try:
            data = self.cfg.get(section, option)
            #print ("DATA    =", data)
        except:
            if os.getenv('DEBUG') or os.getenv('DEBUG_SERVER'):
                traceback.format_exc()
            self.write_config(section, option, filename, value)
            data = configset.cfg.get(section, option)
        return data
        
    def read_config2(self, section, option, filename='', verbosity=None): #format ['aaa','bbb','ccc','ddd']
        """
            option: section, option, filename=''
            format result: ['aaa','bbb','ccc','ddd']
            
        """
        
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        cfg = ConfigParser.RawConfigParser(allow_no_value=True, dict_type=MultiOrderedDict) 
        cfg.read(filename)
        cfg = cfg.get(section, option)
        return cfg
    
    def read_config3(self, section, option, filename='', verbosity=None): #format result: [[aaa.bbb.ccc.ddd, eee.fff.ggg.hhh], qqq.xxx.yyy.zzz]
        """
            option: section, option, filename=''
            format result: [[aaa.bbb.ccc.ddd, eee.fff.ggg.hhh], qqq.xxx.yyy.zzz]
            
        """
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        data = []
        cfg = ConfigParser.RawConfigParser(allow_no_value=True, dict_type=MultiOrderedDict) 
        cfg.read(filename)
        cfg = cfg.get(section, option)
        for i in cfg:
            if "," in i:
                d1 = str(i).split(",")
                d2 = []
                for j in d1:
                    d2.append(str(j).strip())
                data.append(d2)
            else:
                data.append(i)
        return data
    
    def read_config4(self, section, option, filename='', value = '', verbosity=None): #format result: [aaa.bbb.ccc.ddd, eee.fff.ggg.hhh, qqq.xxx.yyy.zzz]
        """
            option: section, option, filename=''
            format result: [aaa.bbb.ccc.ddd, eee.fff.ggg.hhh, qqq.xxx.yyy.zzz]
            
        """
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        #debug(filename = filename)
        data = []
        cfg = ConfigParser.RawConfigParser(allow_no_value=True, dict_type=MultiOrderedDict)
        #print "CFG =", cfg
        cfg.read(filename)
        #debug(cfg0 = cfg)
        try:
            cfg = cfg.get(section, option)
            #debug(cfg1 = cfg)
            if not cfg == None:
                for i in cfg:
                    if "," in i:
                        d1 = str(i).split(",")
                        for j in d1:
                            data.append(str(j).strip())
                    else:
                        data.append(i)
                return data
            else:
                return None
        except:
            #debug(ERROR = traceback.format_exc())
            #debug(except_has = True)
            data = self.write_config(section, option, filename, value, cfg)
            #debug(data = data)
            return data
    
    def read_config5(self, section, option, filename='', verbosity=None): #format result: {aaa:bbb, ccc:ddd, eee:fff, ggg:hhh, qqq:xxx, yyy:zzz}
        """
            option: section, option, filename=''
            format result: {aaa:bbb, ccc:ddd, eee:fff, ggg:hhh, qqq:xxx, yyy:zzz}
            
        """
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        data = {}
        cfg = ConfigParser.RawConfigParser(allow_no_value=True, dict_type=MultiOrderedDict) 
        configset.cfg.read(filename)
        cfg = configset.cfg.get(section, option)
        for i in cfg:
            if "," in i:
                d1 = str(i).split(",")
                for j in d1:
                    d2 = str(j).split(":")
                    data.update({str(d2[0]).strip():int(str(d2[1]).strip())})
            else:
                for x in i:
                    e1 = str(i).split(":")
                    data.update({str(e1[0]).strip():int(str(e1[1]).strip())})
        return data
    
    def read_config6(self, section, option, filename='', verbosity=None): #format result: {aaa:[bbb, ccc], ddd:[eee, fff], ggg:[hhh, qqq], xxx:[yyy:zzz]}
        """
            
            option: section, option, filename=''
            format result: {aaa:bbb, ccc:ddd, eee:fff, ggg:hhh, qqq:xxx, yyy:zzz}
            
        """
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        data = {}
        cfg = ConfigParser.RawConfigParser(allow_no_value=True, dict_type=MultiOrderedDict) 
        configset.cfg.read(filename)
        cfg = configset.cfg.get(section, option)
        for i in cfg:
            if ":" in i:
                d1 = str(i).split(":")
                d2 = int(str(d1[0]).strip())
                for j in d1[1]:
                    d3 = re.split("['|','|']", d1[1])
                    d4 = str(d3[1]).strip()
                    d5 = str(d3[-2]).strip()
                    data.update({d2:[d4, d5]})
            else:
                pass    
        return data
        
    def get_config(self, section, option, filename='', value=None, verbosity=None):
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        try:
            data = self.read_config(section, option, filename, value)
        except ConfigParser.NoSectionError:
            print (traceback.format_exc())
            self.write_config(section, option, filename, value)
            data = self.read_config(section, option, filename, value)
        except ConfigParser.NoOptionError:
            print (traceback.format_exc())
            self.write_config(section, option, filename, value)
            data = self.read_config(section, option, filename, value)
        return data
    
    def get_config2(self, section, option, filename='', value=None, verbosity=None):
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        try:
            data = self.read_config2(section, option, filename)
        except ConfigParser.NoSectionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config2(section, option, filename)
        except ConfigParser.NoOptionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config2(section, option, filename)
        return data
    
    def get_config3(self, section, option, filename='', value=None, verbosity=None):
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        try:
            data = self.read_config3(section, option, filename)
        except ConfigParser.NoSectionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config3(section, option, filename)
        except ConfigParser.NoOptionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config3(section, option, filename)
        return data
    
    def get_config4(self, section, option, filename='', value='', verbosity=None):
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        try:
            data = self.read_config4(section, option, filename)
        except ConfigParser.NoSectionError:
            #print "Error 1 =", traceback.format_exc()
            self.write_config(section, option, value)
            data = self.read_config4(section, option, filename)
            #print "data 1 =", data
        except ConfigParser.NoOptionError:
            #print "Error 2 =", traceback.format_exc()
            self.write_config(section, option, value)
            data = self.read_config4(section, option, filename)
            #print "data 2 =", data
        #print "DATA =", data
        return data
    
    def get_config5(self, section, option, filename='', value=None, verbosity=None):
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        try:
            data = self.read_config5(section, option, filename)
        except ConfigParser.NoSectionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config5(section, option, filename)
        except ConfigParser.NoOptionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config5(section, option, filename)
        return data
    
    def get_config6(self, section, option, filename='', value=None, verbosity=None):
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
        try:
            data = self.read_config6(section, option, filename)
        except ConfigParser.NoSectionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config6(section, option, filename)
        except ConfigParser.NoOptionError:
            print (traceback.format_exc())
            self.write_config(section, option, value)
            data = self.read_config6(section, option, filename)
        return data
    
    def write_all_config(self, filename='', verbosity=None):
        if not os.path.isfile(self.configname):
            filename = self.get_config_file(filename, verbosity)
        else:
            filename = self.configname
    
    def read_all_config(self, filename='', section=[], verbosity=None):
        if not os.path.isfile(self.configname):
            filecfg = self.get_config_file(filename, verbosity)
        else:
            filecfg = self.configname
        configset.cfg.read(filecfg)    
        data = {}
        dbank = []
        if len(section) != 0:
            for x in configset.cfg.options(section):
                d = configset.cfg.get(section, x)
                data.update({x:d})
            dbank.append([section,data])        
        else:    
            #print "configset.cfg.sections() =", configset.cfg.sections()
            for i in configset.cfg.sections():
                section.append(i)
                for x in configset.cfg.options(i):
                    d = configset.cfg.get(i, x)
                    data.update({x:d})
                dbank.append([i,data])
        #print "dbank =",  dbank
        return dbank
        
    def read_all_section(self, filename='', section='server', verbosity=None):
        if not os.path.isfile(self.configname):
            filecfg = self.get_config_file(filename, verbosity)
        else:
            filecfg = self.configname
        configset.cfg.read(filecfg)    
        dbank = []
        dhost = []
        for x in configset.cfg.options(section):
            d = configset.cfg.get(section, x)
            #data.update({x:d})
            dbank.append(d)
            if d:
                if ":" in d:
                    data = str(d).split(":")
                    host = str(data[0]).strip()
                    port = int(str(data[1]).strip())
                    dhost.append([host,  port])
        #print "dbank =",  dbank
        #print "dhost =",  dhost
        return [dhost,  dbank]
            
    def test(verbosity=None):
        filename = self.get_config_file(verbosity)
        configset.cfg.read(filename)
        data = configset.cfg.sections()
        print (configset.cfg.get('router','host'))
        print (data)
    def usage(self):
        parser = argparse.ArgumentParser(formatter_class= argparse.RawTextHelpFormatter)
        parser.add_argument('CONFIG_FILE', action = 'store', help = 'Config file name path')
        parser.add_argument('-r', '--read', help = 'Read Action', action = 'store_true')
        parser.add_argument('-w', '--write', help = 'Write Action', action = 'store_true')
        parser.add_argument('-s', '--section', help = 'Section Write/Read', action = 'store')
        parser.add_argument('-o', '--option', help = 'Option Write/Read', action = 'store')
        parser.add_argument('-t', '--type', help = 'Type Write/Read', action = 'store', default = 1, type = int)
        if len(sys.argv) == 1:
            print ("\n")
            parser.print_help()
        else:
            print ("\n")
            args = parser.parse_args()
            if args.CONFIG_FILE:
                self.configname =args.CONFIG_FILE
                if args.read:
                    if args.type == 1:
                        if args.section and args.option:
                            self.read_config(args.section, args.option)
                    elif args.type == 2:
                        if args.section and args.option:
                            self.read_config2(args.section, args.option)
                    elif args.type == 3:
                        if args.section and args.option:
                            self.read_config3(args.section, args.option)
                    elif args.type == 4:
                        if args.section and args.option:
                            self.read_config4(args.section, args.option)
                    elif args.type == 5:
                        if args.section and args.option:
                            self.read_config5(args.section, args.option)
                    elif args.type == 6:
                        if args.section and args.option:
                            self.read_config6(args.section, args.option)
                    else:
                        print ("INVALID TYPE !")
                        #debug("INVALID TYPE !")
                        print ("\n")
                        parser.print_help()
                else:
                    print ("Please use '-r' for read or '-w' for write")
                    #debug("Please use '-r' for read or '-w' for write")
                    print ("\n")
                    parser.print_help()
            else:
                print ("NO FILE CONFIG !")
                #debug("NO FILE CONFIG !")
                print ("\n")
                parser.print_help()
    

configset_class = configset()
configset_class.configname = configname
if PATH:
    configset_class.path = PATH 
get_config_file = configset_class.get_config_file
write_config = configset_class.write_config
write_config2 = configset_class.write_config2
read_config = configset_class.read_config
read_config2 = configset_class.read_config2
read_config3 = configset_class.read_config3
read_config4 = configset_class.read_config4
read_config5 = configset_class.read_config5
read_config6 = configset_class.read_config6
get_config = configset_class.get_config
get_config2 = configset_class.get_config2
get_config3 = configset_class.get_config3
get_config4 = configset_class.get_config4
get_config5 = configset_class.get_config5
get_config6 = configset_class.get_config6
write_all_config = configset_class.write_all_config
read_all_config = configset_class.read_all_config
read_all_section = configset_class.read_all_section
test = configset_class.test
usage = configset_class.usage

if __name__ == '__main__':
    usage()